// Add ECMA262-5 string trim if not supported natively
//IE8 Specific Change
if (!('trim' in String.prototype)) {
    String.prototype.trim = function () {
        return this.replace(/^\s+/, '').replace(/\s+$/, '');
    };
}


var RSStudio = function (options) {

    var glbException = null;

    var errorMsgArray = new Array();

    //------------------------------------------------------------------------
    // helper functions
    //------------------------------------------------------------------------
    var getTextWidth = function (txt, fontsize) {
        if ($("#RSStudioAlertTextSize").size() == 0) {
            $("<span id=\"RSStudioAlertTextSize\"></span>").appendTo("body");

            $("#RSStudioAlertTextSize").css("white-space", "nowrap");
            $("#RSStudioAlertTextSize").css("visibility", "hidden");
            $("#RSStudioAlertTextSize").css("font-size", fontsize);

        }
        $("#RSStudioAlertTextSize").html(txt);
        var width = $("#RSStudioAlertTextSize").width();

        var maxWidth = 800;
        if ($("body").width() < maxWidth) {
            maxWidth = $("body").width() - 10;
        }

        if (width < 250)
            width = 250;
        else if (width > maxWidth)
            width = maxWidth;
        return width;

    };

    var indexOf = function (list, search) {
        for (i = 0; i < list.length; i++) {
            if (list[i] == search)
                return i;
        }
        return -1;
    };

    var findInList = function (list, name, search) {
        for (i = 0; i < list.length; i++) {
            if (list[i][name] == search)
                return i;
        }
        return -1;
    };

    var isNumeric = function (selector) {
        var valchar = RSStudioPage.LangTrans.numericChars;
        var theValue = $(selector).val();
        var idx;

        if (theValue == "") {
            return false;
        }

        for (idx = 0; idx < theValue.length; idx++) {
            if (valchar.indexOf(theValue.charAt(idx)) == -1) {
                return false;
            }
        }
        return true;
    };

    var writeConsole = function (msg) {
        try {
            console.log(msg);
        }
        catch (ex) {
        }
    };



    var userAlert = function (msg, callback) {

        if (msg != "") {
            if ($("div.RSStudioAlert").size() == 0) {
                $("<div class=\"RSStudioAlert\" style=\"display:none\"></div>").appendTo("body");
            }


            $("div.RSStudioAlert").html(msg);


            if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {

                $("div.RSStudioAlert").dialog({

                    buttons: {
                        OK: function () {
                            $(this).dialog("close");
                        }
                    },

                    width: getTextWidth(msg, "24px"),

                    close: function () {

                        if ($("[data-setfocus=setfocus]").size() > 0) {
                            $("[data-setfocus=setfocus]").select();
                            $("[data-setfocus=setfocus]").focus();
                            if ($("[data-setfocus=setfocus]").offset() != null) {
                                $("html,body").animate({ scrollTop: $("[data-setfocus=setfocus]").offset().top - 10 }, 500);
                            }

                            $("[data-setfocus=setfocus]").removeAttr("data-setfocus");
                        }

                        $(this).dialog("destroy");
                        if (callback != undefined)
                            callback();
                    },

                    modal: true,

                    title: ui.getAlertTitle()

                });
            } else {

                var buttons = "<div><a href=\"#\" data-rel=\"back\" data-role=\"button\" class=\"ui-btn-ok\">OK</a></div>";

                var dialog = $("div.RSStudioAlert");
				var dialog_text = dialog.text();
				if(dialog_text.indexOf("Error: Internet connection lost")>=0){
						buttons = "<div><button href='#' onclick='window.parent.location.reload();' data-icon='refresh' data-iconpos='right'>Refresh</button></div>"
				}
				dialog.append(buttons);

                ui.openPopup(dialog, {
                    title: ui.getAlertTitle(),
                    afterCloseAction: function (event, ui) {

                        if ($("[data-setfocus=setfocus]").size() > 0) {
                            $("[data-setfocus=setfocus]").select();
                            $("[data-setfocus=setfocus]").focus();
                            if ($("[data-setfocus=setfocus]").offset() != null) {
                                $("html,body").animate({ scrollTop: $("[data-setfocus=setfocus]").offset().top - 10 }, 500);
                            }

                            $("[data-setfocus=setfocus]").removeAttr("data-setfocus");
                        }

                        if (callback != undefined) {
                            callback();
                        }
                    }
                });

            }

        }
    };

    var userConfirm = function (msg, cs, callback) {

        if (msg != "") {
            if ($("div.RSStudioAlert").size() == 0) {
                $("<div class=\"RSStudioAlert\" style=\"display:none\"></div>").appendTo("body");
            }


            $("div.RSStudioAlert").html(msg);

            if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {

                $("div.RSStudioAlert").dialog({

                    buttons: {
                        OK: function () {
                            $(this).dialog("destroy");
                            if (callback == undefined)
                                ajax.submit(cs);
                            else
                                callback(true);
                        },

                        Cancel: function () {
                            $(this).dialog("close");
                        }
                    },

                    modal: true,

                    title: ui.getAlertTitle(),

                    width: getTextWidth(msg, "24px"),

                    close: function () {
                        $(this).dialog("destroy");
                        if (callback != undefined)
                            callback(false);
                    }

                });

                $(".ui-button").css("font-size", ".75em");

            } else {

                var buttons = "<div><a href=\"#\" data-rel=\"back\" data-role=\"button\" class=\"ui-btn-ok\">OK</a>"
                   + "<a href=\"#\" data-rel=\"back\" data-role=\"button\">Cancel</a></div>";

                var dialog = $("div.RSStudioAlert");
                dialog.append(buttons);

                var okClicked = false;
                dialog.off("click", "a[data-rel=back]");
                dialog.on("click", "a[data-rel=back]", function (event, ui) {
                    if ($(event.currentTarget).hasClass("ui-btn-ok")) {
                        okClicked = true;
                    }
                });


                ui.openPopup(dialog, {
                    title:  ui.getAlertTitle(),
                    afterCloseAction: function (event, ui) {
                        if (okClicked) {
                            if (callback == undefined)
                                ajax.submit(cs);
                            else
                                callback(true);
                        }
                    }
                });

            }



        }

    };

    //---------------------------------------------------------------------------
    // namespace: ui
    //
    // public functions:
    //      
    //
    // private functions:
    //
    //---------------------------------------------------------------------------
    var ui = function () {


        var findFirstField = function (fieldName) {

            //look for field
            var field = $("[data-rssfield=" + fieldName + "]")[0];
            if (field != undefined)
                return field;

            //look for matrix displayed as dropdown
            var field = $("select[data-rssmatrix=" + fieldName + "]")[0];
            if (field != undefined)
                return field;

            //find first radio button or checkbox selected in a matrix
            var field = $("[data-rssmatrix=" + fieldName + "] input[data-rowselector]:checked")[0];
            if (field != undefined)
                return field;

            //if no row selected in matrix get first radio button or check box
            var field = $("[data-rssmatrix=" + fieldName + "] input[data-rowselector]")[0];
            if (field != undefined)
                return field;

            //last attempt -- try for a simple input field
            var field = $("input[name=" + fieldName + "]")[0];
            if (field != undefined)
                return field;

            return null;

        };

        var getAlertTitle = function () {

            var metadata = document.getElementsByTagName("meta");
            var len = metadata.length;
            var title = "";
            for (var i = 0; i < len; i++) {
                if (metadata[i].name == 'PageLayoutLabel') {
                    title = metadata[i].content;
                    break;
                }
            }
            return title;
        }

        var objectSetFocus = function (fieldName) {

            var theField = findFirstField(fieldName);
            if (theField != null) {

                try {
                    theType = theField.type;
                    if (theType == "radio" || theType == "checkbox") {
                        theField.focus();
                    }
                    else if (theType == "text" || theType == "textarea") {
                        theField.select();
                        theField.focus();
                    }
                    else {
                        theField.focus();
                    }
                }
                catch (e) {

                }
            }
        };

        var cftSubmitQ = new Array();
        var cftSubmitQId = 0;

        var cftSubmitQueue = function (id, name, value) {

            RSStudio.ajax.lockScreen();

            var obj = {};
            obj.id = id;
            obj.name = name;
            obj.value = value;

            cftSubmitQ.push(obj);

            if (cftSubmitQId == 0) {
                cftSubmitQueueLoop();
            }
        };

        var cftSubmitQueueLoop = function () {

            cftSubmitQId = 0;

            if (pageIsLocked()) {
                cftSubmitQId = setTimeout(function () {
                    cftSubmitQueueLoop();
                }, 0);
            }
            else {

                var obj = cftSubmitQ.shift();

                console.log(cftSubmitQ.length + ":" + obj.value);
                var selector = $("div[data-rssobject_inst=" + obj.id + "] input[name=" + obj.name + "]");
                selector.val(obj.value);
                RSStudio.ui.cftSubmit(obj.id, obj.name, selector);
                RSStudio.ajax.unlockScreen();
                if (cftSubmitQ.length > 0) {
                    cftSubmitQueueLoop();
                }
            }
        };

        var cftSubmit = function (id, name, selector) {

            ajax.lockPage();

            var obj = selector;

            if (obj == undefined) {
                obj = $("div[data-rssobject_inst=" + id + "]");
            }

            var cs = getCurrentScope(obj);

            cs.curScope.PageControl.objectType = "Field";
            cs.curScope.PageControl.objectValue = name;
            cs.curScope.PageControl.objectParent = "";
            cs.curScope.PageControl.objectRow = "";
            cs.curScope.PageControl.objectCol = "";
            cs.curScope.PageControl.branchFlag = "0";

            ajax.submitNoWait(cs);
            ajax.unlockPage();


        };

        var getAppUrl = function (selector) {

            var returnObj = {};
            var url = "WWGREST";
            var cs = getCurrentScope(selector);
            var appPath = "";
            if (!cs.isBody) {

                appPath = cs.curObj.attr("data-pbpath");

                url = "/" + appPath + "/WWGREST";

            }

            var sessionIdx = cs.curScope.PageControl.sessionIdx;
            var sessionKey = cs.curScope.PageControl.sessionKey;
            var queryString = "WWGSessionIdx=" + sessionIdx + "&WWGSessionKey=" + sessionKey;

            returnObj.url = url;
            returnObj.queryString = queryString;
            returnObj.sessionIdx = sessionIdx;
            returnObj.sessionKey = sessionKey;
            returnObj.appPath = appPath;

            return returnObj;

        };

        var objectClick = function (objectType, objectValue, validateData, branchFlag, skipValidation, objectParent, cs) {

            if (cs == undefined) {
                var obj = $("[data-rssobject='" + objectValue + "']")
                if (obj.size() > 0)
                    cs = getCurrentScope(obj);
                else
                    cs = getCurrentScope("body");
            }

            if (validateData == undefined)
                validateData = true;

            if (branchFlag == undefined)
                branchFlag = "0";

            if (skipValidation == undefined)
                skipValidation = false;

            if (objectParent == undefined)
                objectParent = "";

            cs.curScope.PageControl.objectType = objectType;
            cs.curScope.PageControl.objectValue = objectValue;
            cs.curScope.PageControl.objectParent = objectParent;
            cs.curScope.PageControl.objectRow = "";
            cs.curScope.PageControl.objectCol = "";
            cs.curScope.PageControl.branchFlag = branchFlag;
            if (pageIsLocked()) {
                return;
            }
            if (!skipValidation) {
                if (validateData || (branchFlag == "1")) {
                    if (!fields.validateFields(cs))
                        return;

                }
            }
            ajax.submit(cs);
        };


        var confirmObjectClick = function (objectType, objectValue, confirmText, validateData, branchFlag, skipValidation, objectParent, cs) {

            if (cs == undefined)
                debugger;

            cs.curScope.PageControl.objectType = objectType;
            cs.curScope.PageControl.objectValue = objectValue;
            cs.curScope.PageControl.objectParent = objectParent;
            cs.curScope.PageControl.objectRow = "";
            cs.curScope.PageControl.objectCol = "";
            cs.curScope.PageControl.branchFlag = branchFlag;

            if (pageIsLocked()) {
                return;
            }
            if (!skipValidation) {
                if (validateData || branchFlag) {
                    if (!fields.validateFields(cs))
                        return;

                }
            }

            userConfirm(confirmText, cs);
        };

        var matrixSort = function () {
            var cs = getCurrentScope(this);
            var fieldName = $(this).attr("data-sortable");
            
            var tab = $(this).closest("table");

            if (tab.hasClass("tablescroll_head")) {
                tab = tab.closest("div.tablescroll").find("table.tablescroll_body")
            }

            var matrixName = tab.attr("data-rssmatrix");

            var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);
            matrixClick("COLUMN", baseMatrixName, 0, fieldName, 0, cs);
        };

        var matrixClick = function (clickType, matrixName, objectRow, objectCol, branchFlag, cs) {

            if (cs == undefined)
                debugger;

            cs.curScope.PageControl.objectType = clickType;
            cs.curScope.PageControl.objectValue = matrixName;
            cs.curScope.PageControl.objectRow = objectRow;
            cs.curScope.PageControl.objectCol = objectCol;
            cs.curScope.PageControl.branchFlag = branchFlag;
            if (pageIsLocked()) {
                return;
            }
            else if (branchFlag) {
                if (!fields.validateFields(cs))
                    return;

            }
            ajax.submit(cs);
        };

        var getCurrentScope = function (obj) {


            var pb = $(obj).closest("span.WWGPBBranch");

            while (pb.length == 0 && $(obj).closest("div.ui-dialog").length != 0) {

                // check to see if we are in a dialog that is part of a pb
                var panelInstance = $(obj).closest("div.ui-dialog-content").attr("data-rssobject_inst");
                var parentName = $(obj).closest("div.ui-dialog-content").attr("data-dialogparent");

                if (parentName == "body") {
                    break;
                }

                obj = $("div[data-anchor=" + panelInstance + "]");
                pb = $("div[data-anchor=" + panelInstance + "]").closest("span.WWGPBBranch[data-rssobject_inst=" + parentName + "]");
            }


            var curScope;
            var curObj;
            var isBody = true;

            if (pb.length != 0) {
                var sessionIdx = pb.attr("data-pbsessionidx");
                curScope = RSStudioPage.PBBranch[pb.attr("data-pbpath") + sessionIdx];
                curObj = pb;
                isBody = false;
            }
            else {
                curScope = RSStudioPage;
                curObj = $("body");
            }


            var ws = curObj.find("div.form_div").filter(":first");


            //add all open dialogs
            if (pb.length != 0) {
                ws = ws.add($("div.ui-dialog-content[data-dialogparent=" + pb.attr("data-rssobject_inst") + "]").closest("div.ui-dialog"));
            } else {
                ws = ws.add($("div.ui-dialog-content[data-dialogparent=body]").closest("div.ui-dialog"));
            }


            var parentCount = ws.parents("span.WWGPBBranch").length;

            var parentArray = new Array();
            ws.parents("span.WWGPBBranch").each(function (idx, value) {
                parentArray.push($(value).attr("data-rssobject_inst"));
            });

            var find = function (selector, e) {
                return ws.find(selector).not(function () {
                    var obj = $(this);

                    var pb = $(obj).closest("span.WWGPBBranch");

                    while (pb.length == 0 && $(obj).closest("div.ui-dialog").length != 0) {
                        var panelInstance = $(obj).closest("div.ui-dialog-content").attr("data-rssobject_inst");
                        var parentName = $(obj).closest("div.ui-dialog-content").attr("data-dialogparent");

                        if (parentName == "body") {
                            break;
                        }

                        obj = $("div[data-anchor=" + panelInstance + "]");
                        pb = $("div[data-anchor=" + panelInstance + "]").closest("span.WWGPBBranch[data-rssobject_inst=" + parentName + "]");

                    }


                    if (obj.parents("span.WWGPBBranch").length != parentArray.length) {
                        return true;
                    }

                    obj.parents("span.WWGPBBranch").each(function (idx, value) {

                        if ($(value).attr("data-rssobject_inst") != parentArray[idx]) {
                            return true;
                        }
                    });

                    return false;

                }).not(e);
            };

            var canFocus = function (obj) {

                //determine if there is a dialog open
                if ($("body").find("div.ui-dialog:visible").length != 0)
                    //if object is in the dialog then we can focus
                    if ($(obj).closest("div.ui-dialog").length != 0)
                        return true;
                    else
                        return false;
                else
                    return true;

            };

            return {
                curScope: curScope,
                curObj: curObj,
                find: find,
                canFocus: canFocus,
                isBody: isBody
            }

        };



        var actionClickEvent = function (e, c) {


            var cs = getCurrentScope(this);

            e.preventDefault();

            var actionName = $(this).attr("data-rssaction");
            var submit = $(this).attr("data-submit");
            var cancel = $(this).attr("data-rsscancel");
            var confirm = $(this).attr("data-rssconfirm");
            var skipConfirm = $(this).attr("data-skipconfirm");
            var ms = $(this).attr("data-matrixstack");

            theMatrix = $(this).closest("table[data-rssmatrix][data-rssmatrix_selector]");
            if (theMatrix.size() > 0) {
                //action is contained in at least one matrix grid
                matrixChangeEventFromObject(this);
            }

            var cancelval = false;
            if (cancel == "true")
                cancelval = true;

            //if the confirm button was clicked in a dialog, the confimation has already happened!
            if (skipConfirm != undefined && skipConfirm == "true") {
                $(this).removeAttr("data-skipconfirm");
                confirm = undefined;
            }

            if (confirm != undefined)
                confirmObjectClick('Button', actionName, confirm, true, submit, cancelval, ms, cs);
            else
                objectClick('Button', actionName, true, submit, cancelval, ms, cs);

            return false;
        };

        var parentChanged = function (selector) {



            var parentDiv = $(selector).closest("div[data-rssobject_inst]");
            while (parentDiv.size() > 0) {
                parentDiv.attr("data-has_changed", "changed");
                parentDiv = parentDiv.parent().closest("div[data-rssobject_inst]");
            }

            var parentSpan = $(selector).closest("span.WWGPBBranch");

            while (parentSpan.size() > 0) {
                parentSpan.attr("data-has_changed", "changed");
                parentSpan = parentSpan.parent().closest("span.WWGPBBranch");
            }

            //if the object is in a panel -- start with its anchor
            var parentPanel = $(selector).closest("div.ui-dialog-content[data-rssobject_inst], div.ui-popup[data-rssobject_inst]");
            if (parentPanel.size()) {
                selector = $("div[data-anchor=" + parentPanel.attr("data-rssobject_inst") + "]");
                parentChanged(selector);
            }


        };

        var fieldChangeEvent = function () {

            var cs = getCurrentScope(this);

            var fieldName = $(this).attr("data-rssfield");

            //if the field is part of a matrix you need to remove the matrix name!
            var baseFieldName = fieldName.substring(fieldName.lastIndexOf("]") + 1);

            //determine if we are inside a matrix then go up chain to mark matrix as changed
            //theMatrix = $(this).closest("table");
            //matrixName = theMatrix.attr("data-rssmatrix");

            parentChanged(this);


            if ($(this).is(":radio") ) {
                var fieldVal = $(this).val();

                cs.find("[data-rssfield=\"" + fieldName + "\"]", this).val([fieldVal]);

                if (RSStudioPage.Branch.pageLayout.responsiveDesign ) {
                    cs.find("[data-rssfield=\"" + fieldName + "\"]").checkboxradio("refresh");
                }
                
            } else if ($(this).is(":checkbox")) {
                var fieldVal = $(this).prop("checked");

                cs.find("[data-rssfield=\"" + fieldName + "\"]", this).prop("checked", fieldVal);

                if (RSStudioPage.Branch.pageLayout.responsiveDesign ) {
                    cs.find("[data-rssfield=\"" + fieldName + "\"]").checkboxradio("refresh");
                }

            }
            else if (cs.curScope.Fields[baseFieldName].fieldType == "Date and Time" && $(this).is("[data-rssdatetime]")) {
                //date and time picker
                if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {

                    var theDate = $(this).data("datepicker").datepicker("getDate");
                    cs.find("[data-rssfield=\"" + fieldName + "\"]", this).each(function () {
                        $(this).data("datepicker").datepicker("setDate", theDate);
                    });
                } else {
                    var fieldVal = $(this).val();
                    cs.find("[data-rssfield=\"" + fieldName + "\"]", this).val(fieldVal);
                }
            } else if ($(this).is("select[data-role=dropdown]")) {
                var fieldVal = $(this).val();
                var html = $(this).siblings("span").html();
                cs.find("[data-rssfield=\"" + fieldName + "\"]", this).val(fieldVal);
                cs.find("[data-rssfield=\"" + fieldName + "\"]", this).each(function (idx, value) {
                    $(value).siblings("span").html(html);
                });

            } else {
                var fieldVal = $(this).val();
                cs.find("[data-rssfield=\"" + fieldName + "\"]", this).val(fieldVal);
            }



            var submit = $(this).attr("data-submit");
            if (submit != undefined) {
                setTimeout(function () {
                    objectClick("field", baseFieldName, true, submit, false, "", cs);
                }, 0);
            }
        };


        var fieldChangeEventRow = function (e, c) {

            var cs = getCurrentScope(this);

            var theField = $(this).closest("table");
            var fieldName = theField.attr("data-rssfield");

            //if the field is part of a matrix you need to remove the matrix name!
            var baseFieldName = fieldName.substring(fieldName.lastIndexOf("]") + 1);

            parentChanged(this);

            var allField = cs.find("table[data-rssfield=\"" + fieldName + "\"][data-rssfield_selector=Row]");

            if (e.type == "selectableselected" && $(c.selected).is("tr")) {
                var selectedRow = c.selected.rowIndex;

                //get number of rows in header
                var hdr = allField.eq(0).find(">thead>tr").size();

                //offset selected row by header and add 1 so that it works with the nth-child
                selectedRow += 1 - hdr;


                allField.find(">tbody>tr[data-rssfield_row=cur]").removeClass("ui-selected");
                allField.find(">tbody>tr[data-rssfield_row=cur]").removeAttr("data-rssfield_row");
                allField.find(">tbody>tr:nth-child(" + selectedRow + ")").attr("data-rssfield_row", "cur");
                allField.find(">tbody>tr:nth-child(" + selectedRow + ")").addClass("ui-selected");
            }
            else if (e.type == "selectableunselected" && $(c.unselected).is("tr")) {

                allField.find(">tbody>tr[data-rssfield_row=cur]").removeClass("ui-selected");
                allField.find(">tbody>tr[data-rssfield_row=cur]").removeAttr("data-rssfield_row");
            }
            else if (e.type == "selectablestop") {

                var submit = theField.attr("data-submit");

                if (submit != undefined) {
                    setTimeout(function () {
                        objectClick("field", baseFieldName, true, submit, false, "", cs);
                    }, 0);
                }
            }
        };



        var fieldChangeEventListview = function (e, c) {


            var cs = getCurrentScope(this);

            var theField = $(this).closest("ul");
            var fieldName = theField.attr("data-rssfield");
            var submit = theField.attr("data-submit");

            //if the field is part of a matrix you need to remove the matrix name!
            var baseFieldName = fieldName.substring(fieldName.lastIndexOf("]") + 1);

            parentChanged(this);

            var allField = cs.find("ul[data-rssfield=\"" + fieldName + "\"][data-rssfield_selector=Listview]");

            var selectedRow = $(e.currentTarget).parent().children("li").index(e.currentTarget) + 1;
            allField.find("li").removeAttr("data-rssfield_row").removeAttr("data-theme").removeClass("ui-btn-up-b").removeClass("ui-btn-hover-b");
            allField.find("li:nth-child(" + selectedRow + ")").attr("data-rssfield_row", "cur").attr("data-theme", "b").addClass("ui-btn-up-b").addClass("ui-btn-hover-b");

            allField.removeAttr("class");
            allField.find("li>a").removeAttr("class");
            allField.removeData("mobileListview");
            //allField.listview("refresh");
            allField.parent("[data-rssobject_inst]").enhanceWithin();


            if (submit != undefined) {
                setTimeout(function () {
                    objectClick("field", baseFieldName, true, submit, false, "", cs);
                }, 0);
            }


        };

        var matrixChangeEventAccordion = function (e, ui) {

            var matrixName;
            var submit;
            var theMatrix = this;

            if (inMatrixChangeEvent)
                return;

            var cs = getCurrentScope(this);

            matrixName = $(this).attr("data-rssmatrix");
            submit = $(this).attr("data-submit");


            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));

            //the index is for each header and panel so you have to divide by 2 to get the correct one.
            var selectedRow = ui.newHeader.index() / 2;


            var allMatrix = cs.find("div[data-rssmatrix='" + matrixName + "']");

            allMatrix.find(">h2").removeAttr("data-rssmatrix_row");

            if (selectedRow >= 0)
                allMatrix.find(">h2:eq(" + selectedRow + ")").attr("data-rssmatrix_row", "cur");

            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                e.preventDefault();

                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                matrixClick("MATRIX", baseMatrixName, "", "", submit, cs);
            }
            else {
                cs.find("span.WWGPBBranch").each(function () {
                    if (ui.newPanel.find(this).size() != 0) {

                        setupPBBranch(this, true);
                    }

                });

                inMatrixChangeEvent = true;
                cs.find("div[data-rssmatrix='" + matrixName + "']").each(function () {
                    if (this != theMatrix)
                        $(this).accordion("option", "active", selectedRow);
                });
                inMatrixChangeEvent = false;
            }

        };

        var matrixChangeEventCollapsable = function (e, ui) {

            var matrixName;
            var submit;
            var theMatrix = $(this).parent("div[data-role=collapsibleset]");

            if (inMatrixChangeEvent)
                return;

            var cs = getCurrentScope(this);

            matrixName = theMatrix.attr("data-rssmatrix");
            submit = theMatrix.attr("data-submit");


            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));


            var expand = e.type == "collapsibleexpand";

            //the index is for each header and panel so you have to divide by 2 to get the correct one.
            var selectedRow = $(e.currentTarget).index();

            var allMatrix = cs.find("div[data-rssmatrix='" + matrixName + "']");
            allMatrix.find(">div[data-role=collapsible]>h2").removeAttr("data-rssmatrix_row");
            allMatrix.find(">div[data-role=collapsible]").attr("data-collapsed", "true");

            if (expand) {
                allMatrix.find(">div[data-role=collapsible]>h2:eq(" + selectedRow + ")").attr("data-rssmatrix_row", "cur");
                allMatrix.find(">div[data-role=collapsible]:eq(" + selectedRow + ")").attr("data-collapsed", "false");
            }

            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                e.preventDefault();

                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                matrixClick("MATRIX", baseMatrixName, "", "", submit, cs);
            }
            else {
                cs.find("span.WWGPBBranch").each(function () {
                    if (ui.newPanel.find(this).size() != 0) {

                        setupPBBranch(this, true);
                    }

                });

                inMatrixChangeEvent = true;
                cs.find("div[data-rssmatrix='" + matrixName + "']").each(function () {
                    if (this != theMatrix) {
                        $(this).find(">div[data-role=collapsible]:eq(" + selectedRow + ")").collapsible("option", "collapsed", false);
                    }
                });
                inMatrixChangeEvent = false;
            }

        };

        var matrixScrollButton = function (e, c) {
            var cs = getCurrentScope(this);
            var matrixName = $(this).attr("name");
            var direction = $(this).attr("data-matrixscrollbutton")
            RSStudio.ui.matrixClick("MatrixScroll", matrixName, direction, "", "0", cs)
        };

        var matrixTabClose = function (e, c) {

            var matrixName;
            var submit;
            var theMatrix;

            var cs = getCurrentScope(this);

            theMatrix = $(this).closest("div.RSSMatrixTabContainer");
            matrixName = theMatrix.attr("data-rssmatrix");
            submit = theMatrix.attr("data-submit");
            if (submit == undefined) {
                //deleteing a row always causes a submit
                submit = 0;
            }
            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));

            var li = $(this).closest("li");

            var selectedRow = li.parent().children("li").index(li) + 1;

            cs.find("div[data-rssmatrix='" + matrixName + "']>ul>li:nth-child(" + selectedRow + ")").attr("data-rssmatrix_row", "del");

            //var panel = cs.find("div[data-rssmatrix='" + matrixName + "']>div.ui-tabs-panel:nth-child(" + (selectedRow + 1) + ")");
            //$.each(panel.find("span.WWGPBBranch"), function (idx, val) {
            //    delete RSStudioPage.PBBranch[$(val).attr("data-pbpath") + $(val).attr("data-pbsessionidx")];
            //});

            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                var childSel = $(li.closest("div").find(">div[id$=_" + selectedRow + "]"));
                matrixTabCloseRemoveChild(childSel);
                matrixClick("MATRIX", baseMatrixName, "", "", submit, cs);
            }
        };

        var matrixTabCloseRemoveChild = function (cs) {

            //recurse through children
            $.each(cs.find("span.WWGPBBranch"), function (index, value) {


                var childcs = ui.getCurrentScope(this);
                matrixTabCloseRemoveChild(childcs);
                sendRemoveSession(value);
            });

        };

        var sendRemoveSession = function (selector) {
            var dbpath = $(selector).attr("data-pbpath");
            var childIdx = $(selector).attr("data-pbsessionidx");

            var pbbranch = dbpath + childIdx;


            var ajaxRequest = $("body").data(pbbranch + "ajaxRequest");

            //if there is a running request then don't send a remove session, just abort the request.  Otherwise you might clear out session data for a request that is really still active on the server
            //the results could be a disaster.  Imaging a query where you have DELETE ALl WHERE id = [field] and the remove session clears the field value.
            if (ajaxRequest != undefined) {
                ajaxRequest.abort();
            } else {


                var data;
                if (RSStudioPage.PBBranch[dbpath + childIdx] == undefined) {
                    var sessionIdx = $(selector).attr("data-pbsessionidx");
                    data = {
                        requestType: "removeSession",
                        WWGSessionIdx: sessionIdx
                    }
                } else {
                    var pc = RSStudioPage.PBBranch[dbpath + childIdx].PageControl;
                    data = {
                        requestType: "removeSession",
                        WWGSessionIdx: pc.sessionIdx,
                        WWGThreadIdx: pc.threadIdx
                    }

                }

                var url = "/" + dbpath + "/" + RSStudioPage.Ajax.infoExch;

                $.ajax({
                    url: url,
                    type: "POST",
                    async: true,
                    dataType: "json",
                    data: data,
                    cache: false,
                    success: function (e, c) {
                        var resp;

                    },
                    error: function (e, c) {
                        var resp;
                    }
                });
            }


        };


        var inMatrixChangeEvent = false;
        var matrixChangeEventTab = function (e, c) {

            var matrixName;
            var submit;
            var theMatrix = this;

            if (inMatrixChangeEvent)
                return;

            var cs = getCurrentScope(this);

            matrixName = $(this).attr("data-rssmatrix");
            submit = $(this).attr("data-submit");

            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));

            var selectedRow = c.newTab.index() + 1;

            cs.find("div[data-rssmatrix='" + matrixName + "'] ul:first-child>li").removeAttr("data-rssmatrix_row");
            cs.find("div[data-rssmatrix='" + matrixName + "'] ul:first-child>li:nth-child(" + selectedRow + ")").attr("data-rssmatrix_row", "cur");

            if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                cs.find("div[data-rssmatrix='" + matrixName + "'] ul:first-child>li>a").removeClass("ui-btn-active");
                cs.find("div[data-rssmatrix='" + matrixName + "'] ul:first-child>li:nth-child(" + selectedRow + ")>a").addClass("ui-btn-active");
            }

            if (submit != undefined) {

                e.preventDefault();

                //if the matrix is part of a matrix you need to remove the parent matrix name!
                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                matrixClick("MATRIX", baseMatrixName, "", "", submit, cs);
            }
            else {
                cs.find("span.WWGPBBranch").each(function () {
                    if (c.newPanel.find(this).size() != 0) {

                        setupPBBranch(this, true);
                    }

                });

                inMatrixChangeEvent = true;
                cs.find("div[data-rssmatrix='" + matrixName + "']").each(function () {
                    if (this != theMatrix)
                        $(this).tabs("option", "active", selectedRow - 1);
                });
                inMatrixChangeEvent = false;
            }
        };


        var matrixChangeEventAnchor = function (e, c) {

            var matrixName;
            var submit;
            var theMatrix;

            e.preventDefault();

            matrixChangeEventFromObject(this);

            var cs = getCurrentScope(this);

            theMatrix = $(this).closest("table[data-rssmatrix][data-rssmatrix_selector]");
            matrixName = theMatrix.attr("data-rssmatrix");

            submit = matrixChangeEventGetSubmit(this);

            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));

            var val = $(this).attr("data-row_idx");
            var colName = $(this).attr("data-column");


            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                matrixClick("MATRIX", baseMatrixName, val, colName, submit, cs);
            }


        };

        var matrixChangeEventDropdown = function (e, c) {

            var matrixName;
            var submit;
            var theMatrix;

            var cs = getCurrentScope(this);

            matrixName = $(this).attr("data-rssmatrix");
            submit = $(this).attr("data-submit");

            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));

            var val = $(this).val();

            if ($(this).is("select[data-role=dropdown]")) {
                var html = $(this).siblings("span").html();

                cs.find("[data-rssmatrix=\"" + matrixName + "\"]", this).val(val);
                cs.find("[data-rssmatrix=\"" + matrixName + "\"]", this).each(function (idx, value) {
                    $(value).siblings("span").html(html);
                });
            } else {
                cs.find("[data-rssmatrix='" + matrixName + "']", this).val(val);
            }

            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                matrixClick("MATRIX", baseMatrixName, val, "", submit, cs);
            }
        };

        //for objects inside a panel matrix we have to select the matrix row
        var matrixChangeEventFromObject = function (theObject) {

            var theParents = $(theObject).parents();

            for (var idx = 0; idx < theParents.size() ; idx++) {

                var value = theParents.eq(idx);

                if ($(value).is("table[data-rssmatrix][data-rssmatrix_selector]")) {

                    var tr = theParents.eq(idx - 2);

                    if ($(tr).is("tr")) {

                        var selector = $(value).attr("data-rssmatrix_selector");

                        //make sure that a submit here does not cause a second submit
                        matrixChangeEventAllowSubmit++;
                        if (selector == "Checkbox" || selector == "Radio Button") {

                            //get the radio button or check box
                            var rb = tr.find("td:first>input:radio,td:first>input:checkbox");

                            //unclick if already checked
                            if (rb.is(':checked')) {
                                rb.prop({ checked: false })
                            }

                            //now click it
                            rb.click();

                        } else if (selector == "Row") {

                            var selectableContainer = theParents.eq(idx - 1);
                            if ($(selectableContainer).is("tbody")) {

                                // add unselecting class to all elements in the styleboard canvas except the ones to select
                                $(".ui-selected", selectableContainer).not(tr).removeClass("ui-selected").addClass("ui-unselecting");
                                //$(".ui-selected", selectableContainer).removeClass("ui-selected");

                                // add ui-selecting class to the elements to select
                                $(tr).removeClass("ui-selected").addClass("ui-selecting");

                                // trigger the mouse stop event (this will select all .ui-selecting elements, and deselect all .ui-unselecting elements)
                                selectableContainer.data("ui-selectable")._mouseStop(null);
                            }
                        } else if (selector == "No Selector") {

                            var selectableContainer = theParents.eq(idx - 1);

                            var matrixName = $(value).attr("data-rssmatrix");
                            var cs = getCurrentScope(tr);
                            var idx = $(selectableContainer).find(tr).index();

                            cs.find("table[data-rssmatrix='" + matrixName + "']>tbody>tr").removeAttr("data-rssmatrix_row");
                            cs.find("table[data-rssmatrix='" + matrixName + "']>tbody>tr").eq(idx).attr("data-rssmatrix_row", "cur");


                            //do a recursive call since there is nothing to click
                            parentChanged(tr);
                            matrixChangeEventFromObject(tr);

                        }
                        matrixChangeEventAllowSubmit--;
                        break;

                    }

                }

            }


        };

        var matrixChangeEventGetSubmit = function (theObject) {

            var submit;
            if (matrixChangeEventAllowSubmit == 0) {
                submit = $(theObject).attr("data-submit");
                if (submit == undefined) {
                    //if any parent should be sumitted, then setup for submit
                    //this code will only be seen in a panel that is setup for grid select
                    $(theObject).parents().each(function (idx, value) {

                        if ($(value).is("table[data-rssmatrix][data-rssmatrix_selector]")) {
                            if (($(value).attr("data-submit") != undefined) && (submit == undefined)) {
                                submit = $(value).attr("data-submit");
                            }
                        }
                    });
                }

            }
            return submit;

        }

        var matrixChangeEventAllowSubmit = 0;
        var matrixChangeEventGrid = function (e, c) {

            var matrixName;
            var submit;
            var theMatrix;

            var cs = getCurrentScope(this);

            theMatrix = $(this).closest("table[data-rssmatrix][data-rssmatrix_selector]");
            matrixName = theMatrix.attr("data-rssmatrix");

            submit = matrixChangeEventGetSubmit(theMatrix);

            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));

            matrixChangeEventFromObject(theMatrix);

            //val will be the current value that has been checked or unchecked {if check box}
            var val = $(this).val();
            var name = $(this).attr("name");

            //get an array of all rows that are checked
            var valArray = theMatrix.find("input[name=" + name + "]:checked").map(function () {
                return $(this).val();
            }).toArray();

            //select rows in all matrices with the same name
            cs.find("table[data-rssmatrix='" + matrixName + "']>tbody>tr>td input[data-rowselector]").val(valArray);

            //set current row indicator
            cs.find("table[data-rssmatrix='" + matrixName + "']>tbody>tr").removeAttr("data-rssmatrix_row");
            cs.find("table[data-rssmatrix='" + matrixName + "']>tbody>tr>td input[value=" + val + "][data-rowselector]").closest("tr").attr("data-rssmatrix_row", "cur");

            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                matrixClick("MATRIX", baseMatrixName, val, "", submit, cs);
            }

        };
        var matrixChangeEventFieldset = function (e, c) {

            var matrixName;
            var submit;
            var theMatrix;

            var cs = getCurrentScope(this);

            theMatrix = $(this).closest("fieldset[data-rssmatrix]");
            matrixName = theMatrix.attr("data-rssmatrix");

            submit = matrixChangeEventGetSubmit(theMatrix);

            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));

            matrixChangeEventFromObject(theMatrix);

            //val will be the current value that has been checked or unchecked {if check box}
            var val = $(this).val();
            var name = $(this).attr("name");

            //get an array of all rows that are checked
            var valArray = theMatrix.find("input[name=" + name + "]:checked").map(function () {
                return $(this).val();
            }).toArray();

            //select rows in all matrices with the same name
            cs.find("fieldset[data-rssmatrix='" + matrixName + "'] input[data-rowselector]").val(valArray);
            cs.find("fieldset[data-rssmatrix='" + matrixName + "'] input[data-rowselector]").checkboxradio("refresh");

            //set current row indicator
            cs.find("fieldset[data-rssmatrix='" + matrixName + "'] label").removeAttr("data-rssmatrix_row");
            cs.find("fieldset[data-rssmatrix='" + matrixName + "'] input[value=" + val + "][data-rowselector]").siblings("label").attr("data-rssmatrix_row", "cur");

            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                matrixClick("MATRIX", baseMatrixName, val, "", submit, cs);
            }

        };

        var matrixChangeEventRow = function (e, c) {

            var matrixName;
            var submit;
            var theMatrix;

            var cs = getCurrentScope(this);

            theMatrix = $(this).closest("table");
            matrixName = theMatrix.attr("data-rssmatrix");

            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));
            matrixChangeEventFromObject(theMatrix);

            var allMatrix = cs.find("table[data-rssmatrix='" + matrixName + "'][data-rssmatrix_selector=Row]");

            if (e.type == "selectableselected" && $(c.selected).is("tr")) {
                var selectedRow = c.selected.rowIndex;

                //get number of rows in header
                var hdr = allMatrix.eq(0).find(">thead>tr").size();

                //offset selected row by header and add 1 so that it works with the nth-child
                selectedRow += 1 - hdr;


                allMatrix.find(">tbody>tr[data-rssmatrix_row=cur]").attr("data-rssmatrix_row", "sel");
                allMatrix.find(">tbody>tr:nth-child(" + selectedRow + ")").attr("data-rssmatrix_row", "cur");
                allMatrix.find(">tbody>tr:nth-child(" + selectedRow + ")").addClass("ui-selected");
            }
            else if (e.type == "selectableunselected" && $(c.unselected).is("tr")) {
                var selectedRow = c.unselected.rowIndex;

                //get number of rows in header
                var hdr = allMatrix.eq(0).find(">thead>tr").size();

                //offset selected row by header and add 1 so that it works with the nth-child
                selectedRow += 1 - hdr;
                allMatrix.find(">tbody>tr:nth-child(" + selectedRow + ")").removeAttr("data-rssmatrix_row");
                allMatrix.find(">tbody>tr:nth-child(" + selectedRow + ")").removeClass("ui-selected");
            }
            else if (e.type == "selectablestop" && (matrixChangeEventAllowSubmit == 0)) {

                submit = matrixChangeEventGetSubmit(theMatrix);

                if (submit != undefined) {
                    //if the matrix is part of a matrix you need to remove the parent matrix name!
                    var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                    matrixClick("MATRIX", baseMatrixName, "", "", submit, cs);
                }
            }
        };

        var matrixChangeEventListview = function (e, c) {
			e.stopPropagation();
            var matrixName;
            var submit;
            var theMatrix;

            var cs = getCurrentScope(this);

            theMatrix = $(this).closest("ul");
            matrixName = theMatrix.attr("data-rssmatrix");

            parentChanged(cs.find("[data-rssmatrix='" + matrixName + "']"));
            matrixChangeEventFromObject(theMatrix);

            var allMatrix = cs.find("ul[data-rssmatrix='" + matrixName + "'][data-rssmatrix_selector=Listview]");

            var selectedRow = $(e.currentTarget).parent().children("li").index(e.currentTarget) + 1;

            //if only one row can be selected

            if (e.ctrlKey) {
                //if on sel or cur change to not selected
                //else change cur row to sel and clicked row to cur
                if (allMatrix.find("li:nth-child(" + selectedRow + ")").eq(0).is("[data-rssmatrix_row]")) {
                    allMatrix.find("li:nth-child(" + selectedRow + ")").removeAttr("data-rssmatrix_row").removeAttr("data-theme").removeClass("ui-btn-up-b").removeClass("ui-btn-hover-b").removeClass("ui-btn-up-c").removeClass("ui-btn-hover-c");

                } else {
                    allMatrix.find("li[data-rssmatrix_row=cur]").attr("data-rssmatrix_row", "sel").attr("data-theme", "c").removeClass("ui-btn-up-b").removeClass("ui-btn-hover-b").addClass("ui-btn-up-c").addClass("ui-btn-hover-c");
                    allMatrix.find("li:nth-child(" + selectedRow + ")").attr("data-rssmatrix_row", "cur").attr("data-theme", "b").addClass("ui-btn-up-b").addClass("ui-btn-hover-b");
                }
                
            } else {
                //change all rows to not selected and clicked row to cur
                allMatrix.find("li").removeAttr("data-rssmatrix_row").removeAttr("data-theme").removeClass("ui-btn-up-b").removeClass("ui-btn-hover-b");
                allMatrix.find("li:nth-child(" + selectedRow + ")").attr("data-rssmatrix_row", "cur").attr("data-theme", "b").addClass("ui-btn-up-b").addClass("ui-btn-hover-b");
            }


            allMatrix.removeAttr("class");
            allMatrix.find("li>a").removeAttr("class");
            allMatrix.removeData("mobileListview");
            allMatrix.parent("[data-rssobject_inst]").enhanceWithin();


            submit = matrixChangeEventGetSubmit(theMatrix);

            if (submit != undefined) {
                //if the matrix is part of a matrix you need to remove the parent matrix name!
                var baseMatrixName = matrixName.substring(matrixName.lastIndexOf("]") + 1);

                //matrixClick("MATRIX", baseMatrixName, "", "", submit, cs);
				setTimeout(function(){matrixClick("MATRIX", baseMatrixName, "", "", submit, cs);},100);	
            }
        };


        var setupContainers = function () {

            //wrap container areas
            $("div.rss_container_area").addClass("ui-corner-all");

            //setup footers
            $("div.rss_footer").addClass("ui-corner-top");

            //set up title areas.
            $("div.rss_title_area").addClass("ui-corner-top");

            //set up the main title
            $("div.rss_main_title").addClass("ui-corner-all");

        };

        var setupButtons = function () {

            if (!$("div.rss_button_area").hasClass("ui-corner-all")) {
                $("div.rss_button_area").addClass("ui-corner-all");
                $("<div style='clear:both'></div>").appendTo("div.rss_button_area");
            }

            $("div.rss_button_area").each(function () {
                var div = $(this);
                div.find("input[type=submit]").each(function (i, inp) {
                    if ($(inp).css("float") != "right")
                        div.prepend(inp);
                });
            });
            var buttons = $("div.rss_button_area input[type=submit]").button().css("float", "right");

            $("input[type=submit].rss_button").button();
        };

        var setupDialogsRd = function () {

            $("div[data-rssdialog=true]").each(function (idx, value) {
                var dialog = $(value);

                if (dialog.html() == "" && dialog.parent("div").hasClass("ui-popup-active")) {
                    dialog.attr("data-dialogclosed", true);
                    dialog.popup("close");
                }
            });

            $("div[data-rssdialog=true]").each(function (idx, value) {


                var dialog = $(value);

                if (dialog.html() == "" || dialog.parent("div").hasClass("ui-popup-active") || dialog.is("[data-popupstack]")) {
                    return;
                }

                if (dialog.attr("data-dialogparent") == undefined) {

                    var cs = getCurrentScope(dialog);

                    if (cs.isBody) {
                        dialog.attr("data-dialogparent", "body");
                    } else {
                        dialog.attr("data-dialogparent", cs.curObj.attr("data-rssobject_inst"));
                    }
                }



                var panelInst = dialog.attr("data-rssobject_inst");
                var anchorName = panelInst + "DialogAnchor";
                if ($("[data-anchor=" + panelInst + "]").size() == 0)
                    dialog.before("<div id=" + anchorName + " data-anchor=\"" + panelInst + "\"></div>");


                var isModal = dialog.attr("data-rssdialog_modal") == "false" ? false : true;
                var closeButton = dialog.attr("data-rssdialog_close") == "false" ? false : true;

                var closeActions = dialog.find("input[name^=WWGAction_][data-rsscancel]");

                if (closeButton && closeActions.size() > 0) {
                    dialog.append("<a href=\"#\" data-rel=\"back\" class=\"ui-btn ui-corner-all ui-shadow ui-btn-a ui-icon-delete ui-btn-icon-notext ui-btn-right\">Close</a>");
                }

                //if there is no close action don't allow isModal.
                if (!isModal && closeActions.size() == 0) {
                    isModal = true;
                }

 

                openPopup(dialog, {
                    dismissible: !isModal,
                    title: dialog.find("span[data-rssdialogtitle]").html(),
                    afterCloseAction: function (event, ui) {
                        if( !dialog.is("[data-dialogclosed]") ) {
                            dialog.find("input[name^=WWGAction_][data-rsscancel]").eq(0).click();
                        } else {
                            dialog.removeAttr("data-dialogclosed");
                        }
                        $("#" + anchorName).remove();
                    }
                });
            });
        };

        var popupStack = new Array();
        var popupIsStacking = false;
        var openPopup = function (dialog, options) {

            if (options == undefined) {
                options = {};
            }

            dialog.attr("data-role", "popup");

            if (options.title != undefined) {
                var html = "<div data-role=\"header\" class=\"ui-content\">" + options.title + "</div><div data-role=\"main\" class=\"ui-content\">" + dialog.html() + "</div>";
                dialog.html(html);
            } else {
                dialog.addClass("ui-content");
            }

            var overlayTheme = "a";
            if (options.dismissible == undefined || !options.dismissible) {
                dialog.attr("data-dismissible", "false");
                overlayTheme = "c";

            }

           
            dialog.popup({
                theme: "a",
                overlayTheme: overlayTheme,
                afterclose: function (event, ui) {
                    if (!popupIsStacking) {
                        dialog.popup("destroy");
                        dialog.css("display", "none");
                        if (options.afterCloseAction != undefined) {
                            options.afterCloseAction();
                        }

                        popupStack.pop();
                        if (popupStack.length > 0) {
                            setTimeout(function () {
                                var oldDialog = popupStack[popupStack.length - 1];
                                oldDialog.removeAttr("data-popupstack");
                                oldDialog.popup("open");
                            }, 1);
                        }

                    } else {
                        popupIsStacking = false;
                    }
                }
            });

            if (popupStack.length > 0) {
                var curDialog = popupStack[popupStack.length - 1];
                popupIsStacking = true;
                curDialog.attr("data-popupstack", "true");
                curDialog.popup("close");
            }

            openPopupWithStack(dialog);
        };

        var openPopupWithStack = function (dialog) {

            if (popupIsStacking) {
                setTimeout(function () {
                    openPopupWithStack(dialog);
                }, 1);
            } else {
                popupStack.push(dialog);
                dialog.css("display", "block");
                dialog.parent().enhanceWithin();
                dialog.popup("open");
            }

        };

        var setupDialogsNonRd = function () {

            //child pbs may strand dialogs if they are refreshed while the dialog is open.  Look for these cases and destroy!
            $("div[data-rssdialog=true]").each(function (e, c) {

                if ($(this).hasClass("ui-dialog-content")) {
                    if ($(this).html() == "" || $("div[data-anchor=" + $(this).attr("data-rssobject_inst") + "]").size() == 0) {
                        try { $(this).dialog("destroy"); } catch (e) {; }
                        try { $(this).dialogExtend("destroy"); } catch (e) { }
                    }
                }

            });

            $("div[data-rssdialog=true]").each(function (e, c) {

                var dialog = $(this);

                if (dialog.html() == "" || dialog.hasClass("ui-dialog-content")) {
                    return;
                }

                var newButtons = {};
                var closeActionName = "";

                //convert each action:
                dialog.find("input[name^=WWGAction_][data-dialogbutton=true]").each(function (e, c) {

                    var action = $(this);
                    if (action.closest("div[data-rssdialog=true]").is(dialog)) {

                        var buttonName = action.attr("name").substring(10);
                        newButtons[buttonName] = {};
                        newButtons[buttonName].value = action.val();
                        newButtons[buttonName].confirm = action.attr("data-rssconfirm");

                        //grab last button set for cancel "skip validation" as the close button
                        if (action.attr("data-rsscancel") == "true") {
                            closeActionName = buttonName;
                        }
                        action.hide();
                    }

                });

                var thewidth = dialog.attr("data-rssdialog_width")*1;
                var theheight = dialog.attr("data-rssdialog_height")*1;
                var thetitle = dialog.children("span[data-rssdialogtitle]").html();
                var isModal = dialog.attr("data-rssdialog_modal");
                var minMaxbuttons = dialog.attr("data-rssdialog_minmax");
                var closeButton = dialog.attr("data-rssdialog_close");
                var allowResize = dialog.attr("data-rssdialog_allowresize");
                var isDraggable = dialog.attr("data-rssdialog_draggable");
                var xPos = dialog.attr("data-rssdialog_xpos")*1;
                var yPos = dialog.attr("data-rssdialog_ypos")*1;
                var position = "center";
                if (xPos != 0) {
                    if( yPos != 0) {
                        position = "left+" + (xPos + (thewidth/2)) + " top+" + (yPos + (theheight / 2));
                    } else {
                        position = "left+" + (xPos + (thewidth / 2)) + " center";
                    }
                } else if (yPos != 0) {
                    position = "center" + " top+" + (yPos + (theheight / 2));
                }


                var collapsable = minMaxbuttons == "true" ? true : false;

                if (thewidth == "0") {
                    thewidth = "auto";
                    collapsable = false; //can't have collabsable and auto set on width
                }

                if (theheight == "0") {
                    theheight = "auto";
                }

                var dialogExtendOptions = {
                    "closable": closeButton == "false" ? false : closeActionName == "" ? false : true,
                    "maximizable": minMaxbuttons == "true" ? true : false,
                    "minimizable": minMaxbuttons == "true" ? true : false,
                    "minimizeLocation": "left",
                    "collapsable": collapsable,
                    "dblclick": "minimize",
                    "titlebar": false
                };

                //if there is no close action don't allow isModal.
                if (!isModal && closeActionName == "") {
                    isModal = true;
                }

                createDialog(
                    dialog,
                    {

                        closeAction: closeActionName,
                        actions: newButtons,

                        dialogOptions: {
                            minWidth: 200,
                            width: thewidth,
                            height: theheight,
                            title: thetitle,
                            modal: isModal == "false" ? false : true,
                            draggable: isDraggable == "false" ? false : true,
                            resizable: allowResize == "false" ? false : true,
                            position: { my: "center", at: position, of: window }
                        }


                    }, 
                    dialogExtendOptions);
            });
        };

        var setupDatetimeNonRd = function () {


            var locale = RSStudioPage.LangTrans.langTransStr;

            var jslocale = "";
            if (locale != undefined && locale != "en-US") {
                jslocale = locale.substring(0, 2);
            }

            if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {
                $.datepicker.setDefaults($.datepicker.regional[jslocale]);
            }

            $("input[data-rssdatetime]").not("[data-rsshaspicker]").each(function () {

                $(this).attr("data-rsshaspicker", "true");

                var name = $(this).attr("name");
                var value = $(this).val();
                //var theDateStr = $(this).attr("data-rssdatetime_date");
                //var theDate = new Date(value.substring(0, 4), value.substring(5, 7) * 1 - 1, value.substring(8, 10), value.substring(11, 13), value.substring(14, 16), value.substring(17));


                var formatParts = $.parseJSON($(this).attr("data-rssdatetime_format"));

                var calType = $(this).attr("data-rssdatetime");
                var objType = "input";
                if (calType == "Fixed")
                    objType = "div";


                $(this).hide();

                var dateInp = $("<" + objType + " name='" + name + "_Datepicker'/>").insertAfter(this);

                $(this).data("datepicker", dateInp);

                var opts = {
                    dateFormat: formatParts.dateFormat,
                    altField: this,
                    //showOn: "button",
                    //buttonImage: "images/cal.gif",
                    //buttonImageOnly: true,
                    onSelect: function (e, c) {
                        var field = $(this).datepicker("option", "altField");
                        $(field).trigger("change");
                    }
                };

                if (formatParts.timeFormat != "") {
                    var amflag = formatParts.timeFormat.substring(formatParts.timeFormat.length - 2) == "TT";

                    $.extend(opts, {
                        timeFormat: formatParts.timeFormat,
                        ampm: amflag,
                        altFieldTimeOnly: false,
                        stepMinute: 5,
                        showButtonPanel: false,
                        controlType: "select",
                        oneLine: true
                    });

                    dateInp.datetimepicker(opts);
                }
                else {
                    dateInp.datepicker(opts);
                }

                if (value != "")
                    dateInp.datepicker("setDate", value);

                //var altField = this;
                //dateInp.change(function () {
				//	debugger;
                //    if (!$(this).val())
                //        $(altField).val('');
                //});

            });
        };

        var setupDatetimeRd = function () {


            $("input[data-rssdatetime]").not("[type=date]").each(function () {


                $(this).val($(this).attr("data-rssdatetime_date").substring(0, 10));
                $(this).attr("type", "date");

                //var mydate = $(this);
                //var name = mydate.attr("name");
                //setTimeout(function () {
                //    var thedate = $("input[name=" + name + "]");
                //    thedate.val(thedate.attr("data-rssdatetime_date").substring(0, 10));
                //    thedate.attr("type", "date");
                //}, 1000);




            });

            //$("body").off("change", "input[data-rssdatetime]");
            //$("body").on("change", "input[data-rssdatetime]", fieldChangeEvent);

        };


        var setupObjectsCommon = function () {

            setupMatrixRadioButton();
            setupFieldSessionKey();


            $("table[data-scrollable]").each(function () {

                var height = $(this).attr("data-scrollable");
                var border = $(this).attr("border");

                $(this).closest("[aria-hidden='true']").show();
                if (!($(this).hasClass("tablescroll_body"))) {
                    $(this).tableScroll({ height: height });
                }

                if ($(this).hasClass("tablescroll_body")) {
                    if (border == "1") {
                        $(this).closest("div.tablescroll").find("table.tablescroll_head").attr("border", "1");
                    }

                    if ($(this).find("[data-rssmatrix_row=cur]").offset() != null) {
                        var offset = $(this).find("[data-rssmatrix_row=cur]").offset().top - $(this).offset().top;
                        //$(this).closest("div.tablescroll_wrapper").animate({scrollTop: offset - 10},500);
                        $(this).closest("div.tablescroll_wrapper").scrollTop(offset - 10);
                    }
                }
                $(this).closest("[aria-hidden='true']").hide();
            });

        };

        
        var setupObjectsNonRD = function () {


            //Matrix with row selector
            try {
                $("table[data-rssmatrix][data-rssmatrix_selector=Row]>tbody").selectable("destroy");
            } catch (e) {; }



            $("table[data-rssmatrix][data-rssmatrix_selector=Row]>tbody").selectable({
                filter: "tr",
                selected: matrixChangeEventRow,
                unselected: matrixChangeEventRow,
                stop: matrixChangeEventRow,
                cancel: "a,input[type=submit]"
            });

            $("table[data-rssmatrix][data-rssmatrix_selector=Row]>tbody>tr[data-rssmatrix_row]").addClass("ui-selected");

            //Field with row selector
            try {
                $("table[data-rssfield_selector=Row]>tbody").selectable("destroy");
            } catch (e) {; }



            $("table[data-rssfield_selector=Row]>tbody").selectable({
                filter: "tr",
                selected: fieldChangeEventRow,
                unselected: fieldChangeEventRow,
                stop: fieldChangeEventRow
            });

            $("table[data-rssfield][data-rssfield_selector=Row]>tbody>tr[data-rssfield_row]").addClass("ui-selected");
        };

        var setupObjectsRD = function () {

            $("table").addClass("ui-corner-all").addClass("ui-shadow");
            $("table").removeAttr("border").removeAttr("cellspacing").removeAttr("cellpadding");
            
            setupObjectsRDFieldCB();
            setupObjectsRDFieldRow();
            setupObjectsRDFieldRB();
            setupObjectsRDFieldRBHZ();

            $("select[data-rssfield], select[data-rssmatrix]").not("[data-role=dropdown]").each(function (idx, value) {
                $(value).attr("data-role", "dropdown");
            });

            setupObjectsRDMatrixRBCB();

            //reformat matrix with row selector
            var ltrs = ["a", "b", "c", "d", "e"];
            $("table[data-rssmatrix_selector=Row]").each(function (idx, value) {

                var matrixname = $(value).attr("data-rssmatrix");
                var submit = "";
                var dataicon = "";

                if ($(value).is("[data-submit]")) {
                    submit = " data-submit=\"" + $(value).attr("data-submit") + "\"";
                } else {
                    dataicon = " data-icon=\"false\"";
                }

                var html = "<ul data-role=\"listview\" data-rssmatrix_selector=\"Listview\" data-inset=\"true\" data-rssmatrix=\"" + matrixname + "\"" + submit + ">";


                $(value).find(">tbody>tr").each(function (tridx, trvalue) {

                    var row = $(trvalue).attr("data-rssmatrix_row");

                    if (row != undefined) {
                        if (row == "sel") {
                            row = " data-rssmatrix_row=\"" + row + "\" data-theme=\"c\" class=\"ui-btn-up-c ui-btn-hover-c\"";
                        } else {
                            row = " data-rssmatrix_row=\"" + row + "\" data-theme=\"b\" class=\"ui-btn-up-b ui-btn-hover-b\"";
                        }
                    } else {
                        row = "";
                    }

                    var td = $(trvalue).find(">td");

                    var newhtml;
                    if (td.size() > 1) {

                        newhtml = "<div class=\"ui-grid-" + ltrs[td.size() < 5 ? td.size() - 2 : 3] + "\">";
                        $.each(td, function (tdidx, tdvalue) {

                            var ltr;
                            if (tdidx < 5) {

                                newhtml += "<div class=\"ui-block-" + ltrs[tdidx] + "\"><div class=\"ui-bar ui-bar-a\">" + td.eq(tdidx).html() + "</div></div>";

                            }

                        });

                        newhtml += "</div>";
                        html += "<li" + row + dataicon + "><a href=\"#\">" + newhtml + "</a></li>";

                    } else {
                        newhtml = td.eq(0).html();

                        var ul_li = $("<ul><li" + row + dataicon + "><a href=\"#\">" + newhtml + "</a></li></ul>"); //ul is just a dummy html container, will be thrown away
                        var li = ul_li.find("li");

                        var attributes = $(trvalue).prop("attributes");
                        // loop through attributes and apply them
                        $.each(attributes, function () {
                            li.attr(this.name, this.value);
                        });

                        attributes = $(td).prop("attributes");
                        // loop through attributes and apply them
                        $.each(attributes, function () {
                            li.attr(this.name, this.value);
                        });

                        html += ul_li.html();
                    }


                });
                html += "</ul>";
                var div = $(value).parent("[data-rssobject_inst]");
                $(value).replaceWith(html);
                //div.enhanceWithin();


            });

            //$("body").off("touchstart mousedown", "ul[data-rssmatrix_selector=Listview]>li");
            //$("body").on("touchstart mousedown", "ul[data-rssmatrix_selector=Listview]>li", matrixChangeEventListview);
            $("body").off("tap mouseup", "ul[data-rssmatrix_selector=Listview]>li");
            $("body").on("tap mouseup", "ul[data-rssmatrix_selector=Listview]>li", matrixChangeEventListview);
        };

        var setupObjectsRDFieldCB = function () {
            //reformat checkboxes
            //move labels inside div if they exist otherwise create them
            $("input[type=checkbox][data-rssfield]").each(function (idx, value) {
                var span = $(value).closest("span[data-rssobject_inst]")
                var labelText;
                if (span.is("[data-rsslabel]")) {
                    var div = span.children("div.ui-checkbox");
                    //make sure that the checkbox is not already enhanced
                    if (div.size() == 0) {
                        labelText = span.attr("data-rsslabel");
                        span.html("<label data-rsslabel=\"true\">" + labelText + span.html() + "</label>");
                    }
                } else {
                    var label = $(value).closest("label");

                    if (label.size() > 0) {
                        var newspan = $(span[0].outerHTML);
                        labelText = label[0].textContent;
                        newspan.html("<label data-rsslabel=\"true\">" + labelText + newspan.html() + "</label>");
                        newspan.attr("data-rsslabel", labelText);
                        label.replaceWith(newspan);
                    } else {
                        var cs = getCurrentScope(value);
                        var theField = cs.curScope.Fields[$(value).attr("data-rssfield")];
                        labelText = theField.label;
                        span.html("<label data-rsslabel=\"true\">" + labelText + span.html() + "</label>");
                        span.attr("data-rsslabel", labelText);
                    }
                }
            });

        };

        var setupObjectsRDFieldRow = function () {
            //reformat field with row selector
            $("table[data-rssfield_selector=Row]").each(function (idx, value) {

                var fieldname = $(value).attr("data-rssfield");
                var name = $(value).attr("name");
                var submit = "";
                var dataicon = "";
                if ($(value).is("[data-submit]")) {
                    submit = " data-submit=\"" + $(value).attr("data-submit") + "\"";
                } else {
                    dataicon = " data-icon=\"false\"";
                }
                var html = "<ul data-role=\"listview\" data-inset=\"true\" data-rssfield_selector=\"Listview\"" + submit + " data-rssfield=\"" + fieldname + "\" name=\"" + name + "\">";
                $(value).find(">tbody>tr").each(function (tridx, trvalue) {

                    var row = $(trvalue).attr("data-rssfield_row");

                    if (row != undefined) {
                        row = " data-rssfield_row=\"cur\" data-theme=\"b\" class=\"ui-btn-up-b ui-btn-hover-b\"";
                    } else {
                        row = "";
                    }

                    var td = $(trvalue).find("td");

                    html += "<li" + row + dataicon + " data-dbvalue=\"" + td.attr("data-dbvalue") + "\"><a href=\"#\">" + td.html() + "</a></li>";

                });
                html += "</ul>";
                var div = $(value).parent("[data-rssobject]");
                $(value).replaceWith(html);
                //div.enhanceWithin();


            });

            //$("body").off("touchstart mousedown", "ul[data-rssfield_selector=Listview]>li");
            //$("body").on("touchstart mousedown", "ul[data-rssfield_selector=Listview]>li", fieldChangeEventListview);
            $("body").off("tap mouseup", "ul[data-rssfield_selector=Listview]>li");
            $("body").on("tap mouseup", "ul[data-rssfield_selector=Listview]>li", fieldChangeEventListview);
        };

        var setupObjectsRDFieldRB = function () {


            //reformat radio button field
            $("table[data-rssfield_selector=\"Radio Button\"]").each(function(idx, value) {

                var title = $(value).find(">thead>tr>th").html();


                var html = "<fieldset data-role=\"controlgroup\">";
                if (title != undefined) {
                    html += "<legend>" + title + "</legend>";
                }

                $(value).find(">tbody>tr").each(function (tridx, trvalue) {
                    var tds = $(trvalue).find("td");
                    html += "<label>" + tds.eq(1).html();
                    html += tds.eq(0).html();
                    html += "</label>";
                });
                html += "</fieldset>";
                
                $(value).replaceWith(html);
                
            });

        };

        var setupObjectsRDFieldRBHZ = function () {


            //reformat radio button field with horizontal layout
            $("table[data-rssfield_selector=\"Radio Button Hz\"]").each(function (idx, value) {
                var html = "";
				if($(value).find("img[name='WWGImage']").length>0){
					//Display Only RB, Table with CB images
					//Looks like: <img alt="" src="images/checkbox_unchecked.jpg" name="WWGImage">
					//OR		: <img alt="" src="images/checkbox_checked.jpg" name="WWGImage">
					html = $("<div>").append($(value).clone()).html(); //Create a temporary div element to get full table html
				}else{


	                var title = $(value).find(">thead>tr>th").html();
	
	
	                var html = "<fieldset data-role=\"controlgroup\" data-type=\"horizontal\">";
	                
	                if (title != undefined) {
	                    html += "<legend>" + title + "</legend>";
	                }
	
	                var tds = $(value).find(">tbody>tr>td");
	                for (var i = 0; i < tds.size() ; i += 3) {
	                    html += "<label>" + tds.eq(i+1).html();
	                    html += tds.eq(i).html();
	                    html += "</label>";
	                }
	
	                html += "</fieldset>";
				}

                $(value).replaceWith(html);

            });

        };


        var setupObjectsRDMatrixRBCB = function () {

            $("table[data-rssmatrix_selector=\"Radio Button\"], table[data-rssmatrix_selector=\"Checkbox\"]").not("[data-role=table]").each(function (idx, value) {

                if ($(value).find("tbody>tr:first>td").size() > 2 || $(value).is("[data-rsspanel]") ) {
                    setupObjectsRDMatrixRBCBMultiCol($(value));
                } else {
                    setupObjectsRDMatrixRBCBSingleCol($(value));
                }
            });

            $("body").off("change", "fieldset[data-rssmatrix] input[data-rowselector]");
            $("body").on("change", "fieldset[data-rssmatrix] input[data-rowselector]", matrixChangeEventFieldset);
        };

        var setupObjectsRDMatrixRBCBSingleCol = function (selector) {


            var title = selector.find(">thead>tr>th").eq(1).html();


            var matrix = selector.attr("data-rssmatrix");

            var html = "<fieldset data-role=\"controlgroup\" data-rssmatrix=\"" + matrix + "\">";
            if (title != undefined) {
                html += "<legend>" + title + "</legend>";
            }

            selector.find(">tbody>tr").each(function (tridx, trvalue) {

                var rowsel = $(trvalue).attr("data-rssmatrix_row");

                var sel = "";
                if (rowsel != undefined) {
                    sel = " data-rssmatrix_row=\"" + rowsel + "\"";
                }

                var tds = $(trvalue).find("td");
                html += "<label" + sel + ">" + tds.eq(1).html();
                html += tds.eq(0).html();
                html += "</label>";
            });
            html += "</fieldset>";

            selector.replaceWith(html);

        };

        var setupObjectsRDMatrixRBCBMultiCol = function (selector) {

            selector.attr("data-role", "table");
            selector.addClass("table-stroke");

            var usePriority = !selector.is("[data-priority=none]");

            selector.find("tbody>tr>td:first-child").each(function (tdidx, tdvalue) {
				if($(tdvalue).find("img[name='WWGImage']").length>0) return;
                $(tdvalue).removeAttr("width");
                $(tdvalue).css("width", "40px");
                $(tdvalue).css("padding", "10px 0 0 0");
            });

            var priorityFnd = false;
            var theadFnd = false;
            selector.find("thead>tr:first>th").each(function (thidx, thvalue) {
                theadFnd = true;
                if ($(thvalue).is("[data-priority]")) {
                    priorityFnd = true;
                }
            });


            if (usePriority) {
                selector.attr("data-mode", "columntoggle");
                selector.addClass("ui-responsive");

                if (!priorityFnd) {

                    var cnt = 0;
                    selector.find("thead>tr:first>th").each(function (thidx, thvalue) {
                        if (thidx > 1) {
                            cnt++;
                            $(thvalue).attr("data-priority", cnt + "");
                        }

                    });

                }
            } else {

                selector.addClass("rsstable-reflow");
                if (!theadFnd) {
                    //jquery mobile throws an error if there is not a thead and th element in a table
                    cnt = selector.find("tbody>tr:first>td").size();
                    selector.prepend("<thead><tr style=\"display:none\"><th colspan=\"" + cnt + "\"></th></tr></thead>");
                }

            }



        };

        var setupMatrixRadioButton = function () {

            $("table[data-rssmatrix_selector='Radio Button']").each(function (idx, selector) {

                if ($(selector).is("[data-sessionKeyAdded]"))
                    return true;

                var cs = getCurrentScope(selector);
                var sessionKey = cs.curScope.PageControl.sessionKey;

                $(selector).attr("data-sessionKeyAdded", true);
                $(selector).find("input[type=radio]").each(function () {
                    $(this).attr("name", $(this).attr("name") + "_" + sessionKey);
                });


            });

        };

        var setupFieldSessionKey = function () {

            $("[data-rssobject^=WWGField_]").each(function (idx, selector) {

                if ($(selector).is("[data-sessionKeyAdded]"))
                    return true;

                var cs = getCurrentScope(selector);
                var field = cs.curScope.Fields[$(selector).attr("data-rssobject")];

                //if display only need to just return that a field was found but there is nothing to do
                if (field == undefined)
                    return true;

                var sessionKey = cs.curScope.PageControl.sessionKey;
                if (field.fieldType == "Radio Button") {
                    $(selector).attr("data-sessionKeyAdded", true);
                    $(selector).find("input[type=radio]").each(function () {
                        $(this).attr("name", $(this).attr("name") + "_" + sessionKey);
                    });
                }
                else if (field.fieldType == "Custom Field Type") {
                    $(selector).attr("data-sessionKeyAdded", true);
                    $(selector).attr("data-rssobject_inst", $(selector).attr("data-rssobject_inst") + "_" + sessionKey);

                }

            });
        };

        var setupUIElements = function (selector) {


            var processedList = new Array();

            var filter =
	            "table.RSSTabContainer"
	            + ",table.RSSAccordionContainer"
	            + ",table[data-rssmatrix_selector=Tab]"
	            + ",table[data-rssmatrix_selector=Accordion]"
                + ",div[data-anchor]"
	            + ",span.WWGPBBranch"

            $(selector).find(filter).each(function (e, c) {

                //make sure you only process the top level of each element container. Contained elements will be picked up recursivley
                var eFnd = false;
                for (var e = 0; e < processedList.length; e++) {
                    if ($.contains(processedList[e], this)) {
                        eFnd = true;
                        break;
                    }
                }

                if (!eFnd) {
                    processedList.push(this);

                    //totLoops++;

                    var fnd = setupTabs(this);
                    if (!fnd)
                        fnd = setupAccordions(this);

                    if (!fnd)
                        fnd = setupMatrixTabs(this);

                    if (!fnd)
                        fnd = setupMatrixAccordions(this);

                    if (!fnd)
                        fnd = setupPBBranch(this);

                    if (!fnd)
                        setupUIElements(this);

                    if (!fnd)
                        fnd = setupDialogAnchor(this);
                }
            });
        };

        var setupDialogAnchor = function (selector) {
            if ($(selector).is("div[data-anchor]")) {

                var anchor = $(selector).attr("data-anchor");
                setupUIElements($("div.ui-dialog-content[data-rssobject_inst=" + anchor + "]"));
                return true;
            }
            return false;
        };

        var setupTabs = function (selector) {

            if ($(selector).is("table.RSSTabContainer")) {

                var panel = 0;
                var panelCnt = $("div.RSSTabContainer").size() + 1;

                var datarole = "";
                if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                    datarole = " data-role=\"tabs\"";
                }


                var tabList = "<div class=\"RSSTabContainer\"" + datarole + ">";

                if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                    tabList += "<div data-role=\"navbar\">";
                }

                tabList += "<ul>";

                $(selector).find(">tbody>tr:nth-child(1)>td").each(function () {
                    panel++;
                    tabList += "<li><a href=\"#RSSTabPanel_" + panelCnt + "_" + panel + "\">" + $(this).html() + "</a></li>";
                });
                tabList += "</ul>";

                panel = 0;
                $(selector).find(">tbody>tr:nth-child(2)>td").each(function () {
                    panel++;
                    tabList += "<div id=\"RSSTabPanel_" + panelCnt + "_" + panel + "\">" + $(this).html() + "</div>";
                });

                if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                    tabList += "</div>";
                }

                tabList += "</div>";

                var newElements = $(tabList).replaceAll(selector);


                try { newElements.tabs("destroy"); } catch (e) {; }
                newElements.tabs({
                    activate: function (event, ui) {
                        $(ui.newPanel).trigger("RSSTabPanelActivate");
                    },
                    heightStyle: "content"

                });

                setupUIElements(newElements);
                return true;
            }
            return false;

        };


        var setupAccordions = function (selector) {

            if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {
                return setupAccordionsNonRd(selector);
            } else {
                return setupAccordionsRd(selector);
            }
        };

        var setupAccordionsNonRd = function(selector) {

            if ($(selector).is("table.RSSAccordionContainer")) {

                var accList = "<div class=\"RSSAccordionContainer\">";
                $(selector).find(">tbody>tr>td").each(function () {
                    if ($(this).hasClass("RSSAccordionTitle")) {
                        accList += "<h2><a href=\"#\">" + $(this).html() + "</a></h2>";
                    }
                    else if ($(this).hasClass("RSSAccordionContent")) {
                        accList += "<div>" + $(this).html() + "</div>";
                    }
                });
                accList += "</div>";

                var newElements = $(accList).replaceAll(selector);

                try { newElements.accordion("destroy"); } catch (e) {; }
                newElements.accordion({
                    autoHeight: false,
                    collapsible: true,
                    heightStyle : "content",
                    active: false
                });

                setupUIElements(newElements);
                return true;
            }
            return false;

        };

        var setupAccordionsRd = function (selector) {

            if ($(selector).is("table.RSSAccordionContainer")) {

                var accList = "<div class=\"RSSAccordionContainer\" data-role=\"collapsibleset\">";
                $(selector).find(">tbody>tr>td").each(function () {
                    if ($(this).hasClass("RSSAccordionTitle")) {
                        accList += "<div data-role=\"collapsible\"><h3>" + $(this).html() + "</h3>";
                    }
                    else if ($(this).hasClass("RSSAccordionContent")) {
                        accList += $(this).html() + "</div>";
                    }
                });
                accList += "</div>";

                var newElements = $(accList).replaceAll(selector);


                setupUIElements(newElements);
                return true;
            }
            return false;

        };

        var setupTabState = function (selector, pbState) {
            //see if in tabs


            var tabs = $(selector).closest("div.ui-tabs");
            if (tabs.size() > 0) {
                var a = tabs.find("a#" + $(selector).closest(".ui-tabs-panel").attr("aria-labelledby"));
                if (pbState == "start") {
                    if (a.data("text") == undefined || a.data("text") == "") {
                        var text = a.html();
                        a.html("<img src=images/busy.gif style='bottom: 5px; left: 2px; position: absolute;'>&nbsp;&nbsp;" + text);
                        a.data("text", text);
                        // hide close box while spinner is showing

                        //a.parent().find("span.ui-icon-close").hide();

                    }
                }
                else {
                    if (a.data("text") != "") {

                        a.html(a.data("text"));
                        a.data("text", "");
                        //a.parent().find("span.ui-icon-close").show();
                    }
                }
            }
        };

        var sendPBInitRequest = function (selector, pbState) {

            var pb = $(selector).attr("data-rssobject");
            var dbpath = $(selector).attr("data-pbpath");
            var childIdx = $(selector).attr("data-pbsessionidx");

            var url = RSStudioPage.Ajax.infoExch;

            var cs = getCurrentScope($(selector).parent());
            var pc = RSStudioPage.PageControl;
            if (!cs.isBody) {
                var parentPath = cs.curObj.attr("data-pbpath");
                var parentIdx = cs.curObj.attr("data-pbsessionidx");
                pc = RSStudioPage.PBBranch[parentPath + parentIdx].PageControl;
                url = "/" + parentPath + "/" + RSStudioPage.Ajax.infoExch;
            }


            $.ajax({
                url: url,
                type: "POST",
                async: false,
                dataType: "json",
                data: {
                    requestType: "initPBInterface",
                    WWGSessionIdx: pc.sessionIdx,
                    WWGThreadIdx: pc.threadIdx,
                    pbBranchName: pb,
                    pbBranchIdx: childIdx,
                    pbState: pbState
                },
                cache: false,
                success: function (e, c) {
                    var resp;
                    if ($(selector).attr("data-pbinit") != "init") {
                        $(selector).attr("data-pbinit", pbState);
                        setupTabState(selector, pbState);
                    }

                },
                error: function (e, c) {
                    var resp;
                }
            });

        };

        var initCookie = function (selector) {

            var dbpath = $(selector).attr("data-pbpath");

            if (RSStudioPage.ChildPath == undefined)
                RSStudioPage.ChildPath = {};

            //add space for each dbpath independent of sessionidx
            if (RSStudioPage.ChildPath[dbpath] == undefined || !RSStudioPage.ChildPath[dbpath].init) {
                RSStudioPage.ChildPath[dbpath] = { init: true };
                var url = "/" + $(selector).attr("data-pbpath") + "/" + RSStudioPage.Ajax.infoExch;

                $.ajax({
                    url: url,
                    type: "POST",
                    dataType: "json",
                    data: { requestType: "initCookie" },
                    cache: false,
                    async: false,
                    success: function (e, c) {
                        var resp;
                    },
                    error: function (e, c) {
                        var resp;
                    }
                });
            }

        };

        var setupPBBranch = function (selector, fromTab) {


            if (fromTab == undefined)
                fromTab = false;


            if (!fromTab && $(selector).is("span.WWGPBBranch") && $(selector).attr("data-pblocalinit") == "true") {
                waitForJSToLoad(selector);
            }
            else if ($(selector).is("span.WWGPBBranch") && $(selector).attr("data-pblocalinit") != "true") {

                var okToInit = false;

                //if from a tab or accordion select, then we know we can init

                if (fromTab) {
                    okToInit = true;
                } else if ($(selector).attr("data-pbinit") == "init") {
                    okToInit = true;
                } else {

                    var div = $(selector).closest("div.ui-widget-content[id^=RSSMatrix]");

                    if (div.size() == 0) {
                        //not inside a tab or accordion
                        okToInit = true;
                    }
                    else if (!div.is(":hidden")) {
                        okToInit = true;
                    } else if (div.attr("aria-hidden") == "false") {
                        okToInit = true;
                    } else {
                        var tabs = div.closest("div.RSSMatrixTabContainer");
                        var acor = div.closest("div.RSSMatrixAccordionContainer");

                        //if we are not inside a tab or accordion, then init
                        if (tabs.size() == 0 && acor.size() == 0) {
                            okToInit = true;
                        } else {
                            var idx = div.index();

                            //if inside a tab, make sure it is the current tab and then init
                            if (tabs.size() > 0) {
                                if (tabs.find(">ul>li:nth-child(" + idx + ")").attr("data-rssmatrix_row") == "cur") {
                                    okToInit = true;
                                }
                            } else {
                                //othewise determine the accordion index, and if current then init
                                idx = (idx - 1) / 2;
                                if (acor.find(">h2:eq(" + idx + ")").attr("data-rssmatrix_row") == "cur") {
                                    okToInit = true;
                                }
                            }
                        }
                    }
                }

                var sessionIdx = $(selector).attr("data-pbsessionidx");
                var dbpath = $(selector).attr("data-pbpath");
                var pbBranch = dbpath + sessionIdx;
                var ajaxrq = $("body").data(pbBranch + "ajaxRequest");

                if (ajaxrq != undefined) {

                    //this tab is still initializing, however, a second submit has caused us to redraw it.  For now just put the spinner back.  This could be caused by an external submit or someone clicking back on the tab.
                    setupTabState(selector, "start");
                    return true;
                }
                else if (okToInit) {
                    if (($(selector).attr("data-pbinit") == "start")) {
                        //the init has really finished but a second submit has caused the client and server to get out of sync.  The second submit finished before the first initialization so that the html sent to the browser shows us still in an init state
                        //just reset the status
                        $(selector).attr("data-pbinit", "init");
                    }

                    $(selector).attr("data-pblocalinit", "true");

                    var html = $(selector).attr("data-pbparm");
                    //var sessionIdx = $(selector).attr("data-pbsessionidx");

                    html = html == "" ? "" : "&" + html;

                    var url = "/" + $(selector).attr("data-pbpath") + "/WWGStart?WWGSessionIdx=" + sessionIdx + "&WWGPBBranch=true" + html;

                    //this happens on a page refresh or if for some reason the span for the pbbranch was replaced...just get it back!
                    //this also happens on a tab change if a child project boook was behind a tab and is being brough forward for the first time
                    if ($(selector).attr("data-pbinit") == "init") {
                        url = "/" + $(selector).attr("data-pbpath") + "/WWGPage?WWGSessionIdx=" + sessionIdx;

                        var pbPage = RSStudioPage.PBBranch[$(selector).attr("data-pbpath") + sessionIdx]
                        if (pbPage != undefined) {
                            url += "&WWGThreadIdx=" + pbPage.PageControl.threadIdx;
                        }

                    }
                    else {
                        initCookie(selector);
                        sendPBInitRequest(selector, "start");
                    }

                    //closing a tab can change the id so we need to drop the tab number -- the session index will give us what we need
                    var instId = $(selector).attr("data-rssobject_inst").substring(0, $(selector).attr("data-rssobject_inst").length - 1);
                    var inst = "span.WWGPBBranch[data-pbpath=" + dbpath + "][data-pbsessionidx=" + sessionIdx + "]";


                    var ajaxRequest = $.ajax({
                        url: url,
                        type: "GET",
                        async: true,
                        dataType: "xml",
                        //data : html,
                        cache: false,
                        success: function (e, c) {

                            var doc = $(e);

                            //$(inst).html(doc.find("NewPage").text());
                            //var dbpath = $(inst).attr("data-pbpath");
                            // var pbBranch = dbpath + sessionIdx;

                            var messageListNode = getNodeByTagName(getNodeByTagName(e, "WWGServlet"), "RSMessageList");
                            ajax.queueMessages(messageListNode);


                            $("body").removeData(pbBranch + "ajaxRequest");

                            RSStudioPage.PBBranch[pbBranch] = $.parseJSON(doc.find("RSStudioPage").text())
                            RSStudioPage.PBBranch[pbBranch].PageControl.lastSubmit = (new Date).getTime();
                            RSStudioPage.PBBranch[pbBranch].PageControl.dbPath = dbpath;
                            RSStudioPage.PBBranch[pbBranch].PageControl.executePageLoad = true;

                            if ($(inst).attr("data-pbinit") != "init") {
                                sendPBInitRequest(inst, "init");
                            }

                            $(inst).html(doc.find("NewPage").text());


                            setupPBImages(inst, RSStudioPage.PBBranch[pbBranch]);

                            //if messages pending don't show yet
                            if (messageListNode == null)
                                $(inst).show();

                            //since we just put in the content, mark as initialized so you don't have to go back to the server
                            $(inst).attr("data-pblocalinit", "true");

                            var newElements = $(inst).find("div:first");
                            setupUIElements(newElements);

                            loadRemoteContent();

                            completeUISetup();
                            waitForJSToLoad(inst);

                            ajax.processMessages();


                        },
                        error: function (e, c) {
                            var resp;
                            var dbpath = $(inst).attr("data-pbpath");
                            var pbBranch = dbpath + sessionIdx;

                            $("body").removeData(pbBranch + "ajaxRequest");
                        }
                    });

                    var dbpath = $(inst).attr("data-pbpath");
                    var pbBranch = dbpath + sessionIdx;

                    $("body").data(pbBranch + "ajaxRequest", ajaxRequest);

                    //var end = new Date().getTime();
                    //ajaxTime += end - start;

                }
                else {
                    var newElements = $(selector).find("div:first");
                    setupUIElements(newElements);
                }
                return true;
            }
            return false;

        };

        var waitForPBBranchLoopCnt = 0;
        var waitForPBBranch = function (waitParms) {


            var selector = waitParms.selector;

            cs = ui.getCurrentScope($(selector));

            if (cs == undefined || cs.curScope == undefined) {

                if ($(selector).attr("data-pbinit") != "") {

                    writeConsole("Waiting for PB branch:" + selector);
                    waitForPBBranchLoopCnt++;

                    if (waitForPBBranchLoopCnt < 5000) {

                        setTimeout(function () {
                            waitForPBBranch(waitParms);
                        }, 100);

                    } else {
                        waitForPBBranchLoopCnt = 0;
                        writeConsole("Wait for PB branch init more than 5000 times:" + selector);
                        waitParms.error();
                    }
                } else {
                    waitForPBBranchLoopCnt = 0;
                    writeConsole("Attempting to initialize PB branch:" + selector);
                    if (setupPBBranch(selector, true)) {
                        setTimeout(function () {
                            waitForPBBranch(waitParms);
                        }, 1000);
                    } else {
                        writeConsole("Unable to initialize PB for messages:" + selector);
                        waitParms.error();

                    }
                }

            } else {
                waitForPBBranchLoopCnt = 0;
                waitParms.success();
            }

        }


        var setupPBImages = function (selector, curScope) {

            //var dbpath = $(selector).attr("data-pbpath");
            var dbpath = curScope.PageControl.dbPath;

            $(selector).find("img[src]").each(function (e, c) {
                if ($(c).attr("src").substring(0, 7) != "http://"
                && $(c).attr("src").substring(0, 8) != "https://"
                && $(c).attr("src").substring(0, 1) != "/"
                && $(c).attr("src").substring(0, 1) != "#")
                    $(c).attr("src", "/" + dbpath + "/" + $(c).attr("src"));
            });
            $(selector).find("input[type='image'][src]").each(function (e, c) {
                if ($(c).attr("src").substring(0, 7) != "http://"
                && $(c).attr("src").substring(0, 8) != "https://"
                && $(c).attr("src").substring(0, 1) != "/"
                && $(c).attr("src").substring(0, 1) != "#")
                    $(c).attr("src", "/" + dbpath + "/" + $(c).attr("src"));
            });
        };

        var setupMatrixTabs = function (selector) {

            if ($(selector).is("table[data-rssmatrix_selector=Tab]")) {
                var panel = 0;

                var minheight = $(selector).parent("div").css("min-height");

                var classNames = " " + $(selector).attr("class");

                var panelCnt = $("div.RSSMatrixTabContainer").size() + 1;;
                var matrixname = $(selector).attr("data-rssmatrix")
                var submit = $(selector).attr("data-submit")
                var closeable = $(selector).attr("data-closeable");
                var span = "";

                if (closeable != undefined) {
                    $("span.ui-icon-close").css({ "float": "right", "margin": "0.4em 0.2em 0 0", "cursor": "pointer" })
                    span = "<span class=\"ui-icon ui-icon-close\">Remove Tab</span>";
                }

                if (submit != undefined)
                    submit = " data-submit=\"" + submit + "\"";
                else
                    submit = "";

                var datarole = "";
                if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                    datarole = " data-role=\"tabs\"";
                }

                var tabList = "<div class=\"RSSMatrixTabContainer" + classNames + "\" data-rssmatrix=\"" + matrixname + "\"" + submit + " data-rssmatrix_selector=\"Tab\"" + datarole + ">";

                if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                    tabList += "<div data-role=\"navbar\">";
                }
                tabList += "<ul>";
                $(selector).find(">tbody>tr>td:nth-child(1)").each(function () {
                    panel++;

                    var row = $(this).closest("tr").attr("data-rssmatrix_row");
                    var active = "";
                    if (row == "cur") {
                        row = " data-rssmatrix_row=\"cur\"";
                        if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                            active = " class=\"ui-btn-active\"";
                        }

                    } else {
                        row = "";
                    }

                    tabList += "<li" + row + "><a href=\"#RSSMatrixTabPanel_" + panelCnt + "_" + panel + "\"" + active + ">" + $(this).html() + "</a>" + span + "</li>";
                });

                tabList += "</ul>";

                panel = 0;
                $(selector).find(">tbody>tr>td:nth-child(2)").each(function () {
                    panel++;
                    tabList += "<div id=\"RSSMatrixTabPanel_" + panelCnt + "_" + panel + "\">" + $(this).html() + "</div>";
                });

                if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                    tabList += "</div>";
                }

                tabList += "</div>";

                var newElements = $(tabList).replaceAll(selector);
                var tabindex = newElements.find("ul:first-child>li").index(newElements.find("ul:first-child>li[data-rssmatrix_row=cur]"));


                try { newElements.tabs("destroy"); } catch (e) {; }

                newElements.tabs({
                    active: tabindex,
                    beforeActivate: matrixChangeEventTab
                });

                if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {

                    if (panel > 0) {
                        newElements.css("min-height", minheight);
                        //if (newElements.parent() != undefined) {
                        //    newElements.parent().show("slow");
                        //}
                    }
                    else {
                        newElements.css("min-height", "0px");
                        if (newElements.parent() != undefined) {
                            newElements.parent().css("min-height", "0px");
                            newElements.parent().attr("data-lastheight", "");
                            //newElements.parent().hide("fast");
                        }
                    }
                }
                setupUIElements(newElements);
                return true;
            }
            return false;

        };


        var setupMatrixAccordions = function (selector) {
            if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {
                return setupMatrixAccordionsNonRd(selector);
            } else {
                return setupMatrixAccordionsRd(selector);
            }
        };

        var setupMatrixAccordionsNonRd = function(selector) {

            if ($(selector).is("table[data-rssmatrix_selector=Accordion]")) {

                var panel = 0;
                var panelCnt = 0;
                var matrixname = $(selector).attr("data-rssmatrix")
                var submit = $(selector).attr("data-submit")
                if (submit != undefined)
                    submit = " data-submit=\"" + submit + "\"";
                else
                    submit = "";

                var accList = "<div class=\"RSSMatrixAccordionContainer\" data-rssmatrix=\"" + matrixname + "\"" + submit + " data-rssmatrix_selector=\"Accordion\">";
                $(selector).find(">tbody>tr").each(function () {

                    var row = $(this).attr("data-rssmatrix_row");
                    if (row == "cur")
                        row = " data-rssmatrix_row=\"cur\"";
                    else
                        row = "";

                    var title = $(this).find(">td:nth-child(1)").html();
                    var panel = $(this).find(">td:nth-child(2)").html();

                    accList += "<h2" + row + ">" + title + "</h2>";
                    accList += "<div>" + panel + "</div>";
                });
                accList += "</div>";


                var newElements = $(accList).replaceAll(selector);
                var tabindex = newElements.find(">h2").index(newElements.find(">h2[data-rssmatrix_row=cur]"));

                if (tabindex < 0)
                    tabindex = false;

                try { newElements.accordion("destroy"); } catch (e) {; }

                newElements.accordion({
                    active: tabindex,
                    beforeActivate: matrixChangeEventAccordion,
                    collapsible: true,
                    autoHeight: false,
                    heightStyle: "content"
                });

                setupUIElements(newElements);
                return true;
            }
            return false;

        };

        var setupMatrixAccordionsRd = function (selector) {

            if ($(selector).is("table[data-rssmatrix_selector=Accordion]")) {

                var panel = 0;
                var panelCnt = 0;
                var matrixname = $(selector).attr("data-rssmatrix")
                var submit = $(selector).attr("data-submit")
                if (submit != undefined)
                    submit = " data-submit=\"" + submit + "\"";
                else
                    submit = "";

                var accList = "<div class=\"RSSMatrixAccordionContainer\" data-rssmatrix=\"" + matrixname + "\"" + submit + " data-rssmatrix_selector=\"Accordion\" data-role=\"collapsibleset\">";
                $(selector).find(">tbody>tr").each(function () {

                    var row = $(this).attr("data-rssmatrix_row");
                    var heading = " data-collapsed=\"true\"";
                    if (row == "cur") {
                        row = " data-rssmatrix_row=\"cur\"";
                        heading = " data-collapsed=\"false\"";
                    } else {
                        row = "";
                    }

                    var title = $(this).find(">td:nth-child(1)").html();
                    var panel = $(this).find(">td:nth-child(2)").html();

                    accList += "<div data-role=\"collapsible\"" + heading + "><h2" + row + ">" + title + "</h2>";
                    accList += panel + "</div>";
                });
                accList += "</div>";


                var newElements = $(accList).replaceAll(selector);


                newElements.on("collapsiblecollapse", "[data-collapsed=false]", matrixChangeEventCollapsable);

                newElements.on("collapsibleexpand", "[data-collapsed=true]", matrixChangeEventCollapsable);


                setupUIElements(newElements);
                return true;
            }
            return false;

        };

        var newPageSetup = function () {

            //this routine is called when a new page is sent via ajax and we need to cleanup javascript and existing stylesheets.
            //loadedScripts = new Array();
            scriptLoaded = false;
            $("link[data-rsstudio=loaded]").remove();
            //loadedCSS = new Array();

            pageSetup(false);
        };

        var newPBPageSetup = function (cs) {

            //this routine is called when a new PB page is sent via ajax and we need to cleanup javascript and existing stylesheets.
            //loadedScripts = new Array();
            scriptLoaded = false;
            $("link[data-rsstudio=loaded]").remove();
            //loadedCSS = new Array();

            setupPBImages(cs.curObj, cs.curScope);

            ui.setupUI(cs.curObj);
        };


        var pageSetup = function (getFields) {
			
			if (RSStudioPage.Branch.pageLayout.responsiveDesign ){
				//Page setup for mobile apps
				var userAgent = window.navigator.userAgent;
				if(userAgent.indexOf("MSIE")>=0 || userAgent.indexOf("Trident")>=0 || userAgent.indexOf("Edge")>=0){
					//prevent default blur function for popups
					$.widget("mobile.popup", $.mobile.popup, {
						_safelyBlur: function (currentElement) {
							try {
								this._super();
							} catch (ex) {
							}
						}
					});
				}
			}
            if (getFields) {
                updateRSStudioPage();
            }

            RSStudioPage.PageControl.executePageLoad = true;



            setupUI("body");

            // attach event handler for values changing so that we can keep everything in sync
            $("body").off("change", "[data-rssfield]");
            $("body").on("change", "[data-rssfield]", fieldChangeEvent);

            // attach event handler for action button click
            $("body").off("click", "[data-rssaction]");
            $("body").on("click", "[data-rssaction]", actionClickEvent);


            // attach event handler for matrix row changing so that we can keep everything in sync
            $("body").off("change", "table[data-rssmatrix] input[data-rowselector]");
            $("body").on("change", "table[data-rssmatrix] input[data-rowselector]", matrixChangeEventGrid);


            $("body").off("change", "select[data-rssmatrix]");
            $("body").on("change", "select[data-rssmatrix]", matrixChangeEventDropdown);

            // attach event handler for anchors inside of tables
            $("body").off("click", "table[data-rssmatrix] a[data-column]");
            $("body").on("click", "table[data-rssmatrix] a[data-column]", matrixChangeEventAnchor);

            //handle sorting of matrix columns
            $("body").off("click", "th[data-sortable]");
            $("body").on("click", "th[data-sortable]", matrixSort);

            $("body").off("mouseover", "th[data-sortable]");
            $("body").on("mouseover", "th[data-sortable]", function () {
                this.style.cursor = "pointer";
            });

            //add event handler for tab close
            $("body").off("click", "div.RSSMatrixTabContainer  li  span.ui-icon-close");
            $("body").on("click", "div.RSSMatrixTabContainer li  span.ui-icon-close", matrixTabClose);


            //events for scrollable matrix
            $("body").off("click", "input.WWGButton[data-matrixscrollbutton]");
            $("body").on("click", "input.WWGButton[data-matrixscrollbutton]", matrixScrollButton);


            if (RSStudioPage.PageControl.eventsPending.toLowerCase() == 'true') {
                events.listenForEvents();
            }

        };


        var updateRSStudioPage = function () {

            RSStudioPage.PageControl.lastSubmit = (new Date).getTime();

            $.ajax({
                url: RSStudioPage.Ajax.infoExch,
                type: "POST",
                async: false,
                dataType: "json",
                data: {
                    requestType: "getFieldList",
                    WWGSessionIdx: RSStudioPage.PageControl.sessionIdx,
                    WWGThreadIdx: RSStudioPage.PageControl.threadIdx
                },
                cache: false,
                success: function (e, c) {
                    $.extend(RSStudioPage.Fields, e);
                },
                error: function (e, c) {
                    var resp;
                }
            });


        }

        var totTime = 0;
        var ajaxTime = 0;
        var totLoops = 0;

        var setupUI = function (startingPoint) {

            keepAlive.startKeepAliveTimer();

            //var start = new Date().getTime();
            setupUIElements(startingPoint);
            loadRemoteContent();
            completeUISetup();
            //var end = new Date().getTime();
            //totTime += end - start;



            //if this was initiated by a child then call the child also
            if ($(startingPoint).is("span.WWGPBBranch"))
                waitForJSToLoad(startingPoint);
            else
                waitForJSToLoad();

        };

        var loadRemoteContent = function () {
            loadJavaScript();
            loadCSS();
        };

        var completeUISetup = function () {

            if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {
                setupObjectsNonRD();
                $("select[data-issearchable=true]").css("min-width", "200px");
                $("select[data-issearchable=true]").chosen();
                setupDatetimeNonRd();
                setupDialogsNonRd();
                setupContainers();

                //not sure of the purpose of this function
                //setupButtons();

            } else {
                setupObjectsRD();
                setupDialogsRd();
                setupDatetimeRd();

                //prior to loading jquery mobile this flag was set to true and a tag is included in the body so no enhancements take place
                //this turns back on the enhancements
                $.mobile.ignoreContentEnabled = false;
                $("body").enhanceWithin();
				$("html.ui-mobile [data-role='page']>[data-role]").fadeIn(); //show header,main,footer if flash prevention is enabled
            }

            setupObjectsCommon();
        };

        var waitForJSToLoad = function (pbSelector) {

            if (loadJSScriptCnt > 0) {
                setTimeout(function () {
                    waitForJSToLoad(pbSelector);
                }, 10);
            }
            else {
                standardJSDynamic(pbSelector);
                pageLoadJSDynamic(pbSelector);
                setTimeout(function () {
                    waitForAllContent();
                }, 0);
            }
        };

        var waitForAllContent = function () {
            //$("body").show();

            var alertMsg = RSStudioPage.PageControl.alertMessage;

            //copy child PB alert messages
            $.each(RSStudioPage.PBBranch, function (idx, value) {
                if (value.PageControl.alertMessage != "") {
                    if (alertMsg != "") {
                        alertMsg += "<br/>";
                    }
                    alertMsg += value.PageControl.alertMessage;
                    value.PageControl.alertMessage = "";
                }
            });

            RSStudioPage.PageControl.alertMessage = alertMsg;

            if (err.hasErrorsPending()) {
                var alertmsg = err.formatErrorMsg();
                if (RSStudioPage.PageControl.alertMessage != "") {
                    alertmsg += "<br/>" + RSStudioPage.PageControl.alertMessage;
                }
                RSStudio.userAlert(alertmsg, function () {
                    err.clearMessages();
                    RSStudioPage.PageControl.alertMessage = "";
                });
            }
            else
                RSStudio.userAlert(RSStudioPage.PageControl.alertMessage, function () {
                    RSStudioPage.PageControl.alertMessage = "";
                });

            if (RSStudioPage.PageControl.focusField != "") {
                objectSetFocus(RSStudioPage.PageControl.focusField);
                RSStudioPage.PageControl.focusField = "";
            }
        };

        var loadJSScriptCnt = 0;

        var loadJavaScript = function () {




            //load css pb branch fields, pages and page layouts
            $("span.WWGPBBranch").each(function () {
                var dbpath = $(this).attr("data-pbpath");
                var pbpage = RSStudioPage.PBBranch[dbpath + $(this).attr("data-pbsessionidx")];
                if (pbpage != undefined)
                    loadJSFromScope(pbpage, dbpath);
            });

            loadJSFromScope(RSStudioPage);

        };

        var loadJSFromScope = function (curScope, url) {


            var sKey = curScope.PageControl.sessionKey;

            var path = "JavaScript/";
            if (url != undefined)
                path = "/" + url + "/JavaScript/";

            if (curScope.Branch.pageLayout.hasStandardJS)
                getScript(path + curScope.PageControl.projectBookIdent + "/WWGPageLayout_" + curScope.Branch.pageLayout.name + ".js", sKey);

            loadExternalJS(curScope.Branch.pageLayout.customJS, url, sKey);

            if (curScope.Branch.page.hasStandardJS)
                getScript(path + curScope.PageControl.projectBookIdent + "/WWGPage_" + curScope.Branch.page.name + ".js", sKey);

            loadExternalJS(curScope.Branch.page.customJS, url, sKey);

            for (var fld in curScope.Fields) {
                if (curScope.Fields[fld].fieldType == "Custom Field Type") {
                    if (curScope.Fields[fld].hasStandardJS == "true") {
                        //getScript(path + curScope.PageControl.projectBookIdent + "/WWGCFT_" + curScope.Fields[fld].fieldTypeParent + ".js?sessionKey=" + sKey);
                        getScript(path + curScope.PageControl.projectBookIdent + "/WWGCFT_" + curScope.Fields[fld].fieldTypeParent + ".js", sKey);
                    }

                    loadExternalJS(curScope.Fields[fld].customJS, url, sKey);
                }
            }
        };

        var loadExternalJS = function (customjs, url, sKey) {

            var customArray = customjs.split(";");

            $(customArray).each(function (e, c) {
                if (c != "") {
                    if (url != undefined && url != "") {
                        c = "/" + url + "/" + c;
                    }

                    getScript(c, sKey);
                }
            });

        };

        var loadedScripts = new Array();
        var scriptLoaded = false;
        var getScript = function (url, skey) {

            if (indexOf(loadedScripts, url) == -1) {
                loadedScripts.push(url);

                if ($("script[src='" + url + "']").size() != 0)
                    return;

                scriptLoaded = true;
                loadJSScriptCnt++;

                if (skey != undefined) {
                    url += "?key=" + skey

                }

                try {
                    $.ajax({
                        url: url,
                        type: "GET",
                        async: false,
                        dataType: "script",
                        cache: true,
                        success: function (e, c) {
                            loadJSScriptCnt--;
                        },
                        error: function (jqXHR, textStatus, errorThrown) {
                            loadJSScriptCnt--;
                        },
                        global: false
                    });
                }
                catch (e) {
                    err.writeLog("getScript error: url:" + url + " error:" + e);
                    loadJSScriptCnt--;
                }

            }
        };


        var loadCSS = function () {

            //add all loaded css to the list
            //            $("link[data-rsstudio]").each(function() {
            //                var url = $(this).attr("href");  
            //                if( indexOf(loadedCSS, url) == -1 )
            //                {
            //                    loadedCSS.push(url);
            //                }
            //            });


            //load css pb branch fields, pages and page layouts

            $("span.WWGPBBranch").each(function () {
                var dbpath = $(this).attr("data-pbpath");
                var pbpage = RSStudioPage.PBBranch[dbpath + $(this).attr("data-pbsessionidx")];
                if (pbpage != undefined)
                    loadCSSFromScope(pbpage, dbpath);
            });


            loadCSSFromScope(RSStudioPage);

        };

        var loadCSSFromScope = function (curScope, url) {

            if (url != undefined) {
                loadExternalCSS(curScope.Branch.pageLayout.css, url);
                loadExternalCSS(curScope.Branch.page.css, url);
            }
            for (var fld in curScope.Fields) {
                if (curScope.Fields[fld].fieldType == "Custom Field Type") {
                    loadExternalCSS(curScope.Fields[fld].css, url);
                }
            }
        };

        var loadExternalCSS = function (css, url) {
            var cssArray = css.split(";");

            $(cssArray).each(function (e, c) {
                if (c != "") {
                    if (url != undefined && url != "")
                        c = "/" + url + "/" + c;

                    getCSS(c);
                }
            });

        };

        //var loadedCSS = new Array();
        var getCSS = function (url) {

            if ($("link[href='" + url + "']").size() == 0) {
                //loadedCSS.push(url);

                //                var link = $("<link data-rsstudio=\"loaded\">");
                //                link.attr({
                //                        type: 'text/css',
                //                        rel: 'stylesheet',
                //                        href: url
                //                });
                //                $("head").append( link ); 


                var style = document.createElement("link");
                style.setAttribute("type", "text/css");
                style.setAttribute("rel", "stylesheet");
                style.setAttribute("href", url);
                var newLink = $("head")[0].insertBefore(style, $("link[data-rsstudio=parent]")[0]);
                //var newLink = $("link[data-rsstudio=parent]:first").before($(style));
                $(newLink).attr("data-rsstudio", "loaded");
            }
        };

        var standardJSScript = "";
        var standardJSDynamic = function (pbSelector) {

            //set up page validateion
            if ($("div#RSStudioPageStandardJS").size() == 0) {
                $("body").append("<div id=\"RSStudioPageStandardJS\"></div>");
            }

            var script = "<script type=\"text/javascript\" data-scripttype=\"rsstudiovalidate\">function RSStudioPageStandardJS() {\n";

            if (pbSelector != undefined) {

                var dbpath = $(pbSelector).attr("data-pbpath");
                //var pbpage = RSStudioPage.PBBranch[dbpath + $(this).attr("data-pbsessionidx")];
                var cs = getCurrentScope(pbSelector);
                if (cs.curScope != undefined)
                    script += standardJSDynamicFromScope(cs, cs.curScope.PageControl);
            }
            else
                script += standardJSDynamicFromScope(getCurrentScope($("body")), RSStudioPage.PageControl);

            script += "}\n";
            script += "//# sourceURL=RSStudioPageStandardJS.js\n"
            script += "</script>\n";

            if (standardJSScript != script) {
                standardJSScript = script;
                $("div#RSStudioPageStandardJS").html(script);
            }

            RSStudioPageStandardJS();

            if (pbSelector != undefined) {
                var dbpath = $(pbSelector).attr("data-pbpath");
                var pb = RSStudioPage.PBBranch[dbpath + $(pbSelector).attr("data-pbsessionidx")];
                if (pb != undefined)
                    pb.PageControl.executePageLoad = false;
            }
            else
                RSStudioPage.PageControl.executePageLoad = false;
        }

        var standardJSDynamicFromScope = function (cs, pc) {

            var script = "";

            for (var fld in cs.curScope.Fields) {
                var theField = cs.curScope.Fields[fld];

                if (theField.fieldType == "Custom Field Type" && theField.hasStandardJS == "true") {
                    cs.find("[data-rssobject=" + fld + "]").each(function () {

                        var id = $(this).attr("data-rssobject_inst");

                        if (cs.curScope.PageControl.CFTFields == undefined)
                            cs.curScope.PageControl.CFTFields = {};

                        if (cs.curScope.PageControl.CFTFields[id] == undefined)
                            cs.curScope.PageControl.CFTFields[id] = {};

                        var pageField = cs.curScope.PageControl.CFTFields[id];

                        if ($(this).html() != "") {


                            if (pageField.hasBeenInit == undefined || pageField.hasBeenInit != true) {
                                pageField.hasBeenInit = true;
                                script += "\ttry\n\t{\n\t\tRSStudio_CFT_" + theField.fieldTypeParent + ".pageLoad(\"" + id + "\", \"" + fld + "\");\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";
                            }
                            script += "\ttry\n\t{\n\t\tRSStudio_CFT_" + theField.fieldTypeParent + ".render(\"" + id + "\", \"" + fld + "\");\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";
                        }
                        else {
                            pageField.hasBeenInit = false;
                        }
                    });
                }
            }

            if (cs.curScope.Branch.pageLayout.hasStandardJS) {
                if (cs.curScope.PageControl.executePageLoad)
                    script += "\ttry\n\t{\n\t\tRSStudio_PageLayout_" + cs.curScope.Branch.pageLayout.name + ".pageLoad();\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";

                script += "\ttry\n\t{\n\t\tRSStudio_PageLayout_" + cs.curScope.Branch.pageLayout.name + ".render();\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";
            }

            if (cs.curScope.Branch.page.hasStandardJS) {
                if (cs.curScope.PageControl.executePageLoad)
                    script += "\ttry\n\t{\n\t\tRSStudio_Page_" + cs.curScope.Branch.page.name + ".pageLoad();\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";

                script += "\ttry\n\t{\n\t\tRSStudio_Page_" + cs.curScope.Branch.page.name + ".render();\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\t";
            }

            return script;

        };

        var pageLoadJSScript = "";
        var pageLoadJSDynamic = function (pbSelector) {


            //set up page validateion
            if ($("div#RSStudioPageLoadJS").size() == 0) {
                $("body").append("<div id=\"RSStudioPageLoadJS\"></div>");
            }

            var script = "<script type=\"text/javascript\" data-scripttype=\"rsstudiovalidate\">function RSStudioPageLoadJS() {\n";

            if (pbSelector != undefined) {
                var dbpath = $(pbSelector).attr("data-pbpath");
                var pbpage = RSStudioPage.PBBranch[dbpath + $(pbSelector).attr("data-pbsessionidx")];
                if (pbpage != undefined && pbpage.JavaScript.pageLoad != "") {
                    script += "try\n{\n" + pbpage.JavaScript.pageLoad + "\n}\ncatch(e)\n{\n\tRSStudio.err.writeLog(e);\n}\n";
                    pbpage.JavaScript.pageLoad = "";
                }
            }


            if (RSStudioPage.JavaScript.pageLoad != "") {
                script += "try\n{\n" + RSStudioPage.JavaScript.pageLoad + "\n}\ncatch(e)\n{\n\tRSStudio.err.writeLog(e);\n}\n";
                RSStudioPage.JavaScript.pageLoad = "";
            }

            script += "}\n";
            script += "//# sourceURL=RSStudioPageLoadJS.js\n"
            script += "</script>\n";

            if (pageLoadJSScript != script) {
                pageLoadJSScript = script;
                $("div#RSStudioPageLoadJS").html(script);
            }
            RSStudioPageLoadJS();
        };


        var createDialog = function (selector, options, dialogExtendOptions) {

            if ($(selector).html() == "")
                return;

            //add parent info
            var cs = getCurrentScope(selector);

            if ($(selector).attr("data-dialogparent") == undefined) {

                if (cs.isBody) {
                    $(selector).attr("data-dialogparent", "body");
                } else {
                    $(selector).attr("data-dialogparent", cs.curObj.attr("data-rssobject_inst"));
                }
            }

            //var panelName = $(selector).attr("data-rssobject");
            var panelInst = $(selector).attr("data-rssobject_inst");
            var anchorName = panelInst + "DialogAnchor";

            if ($("[data-anchor=" + panelInst + "]").size() == 0)
                $(selector).before("<div id=" + anchorName + " data-anchor=\"" + panelInst + "\"></div>");

            var newButtons = {};
            //set up action buttons
            $.each(options.actions, function (key, value) {

                var buttonName = key;
                var confirm = value.confirm;
                newButtons[value.value] = function (e) {

                    $("#" + anchorName).data("buttonName", key);

                    var dlg = this;
                    if (confirm != undefined) {
                        RSStudio.userConfirm(confirm, undefined, function (userConfirm) {

                            if (userConfirm)
                                $(dlg).dialog("close");
                        });
                    }
                    else {
                        $(this).dialog("close");
                    }
                };
            });

            //make sure there is at least an empty object
            var blankOpts = {
                dialogOptions: {
                    buttons: {}
                }
            };

            options = $.extend(blankOpts, options);

            options.dialogOptions.buttons = $.extend(newButtons, options.dialogOptions.buttons);


            $("#" + anchorName).data("actions", options.actions);

            var defaultOptions = {
                autoOpen: false,
                modal: true,
                open: function (event, ui) {
                    if (options.closeAction != "") {
                        var confirmMsg = options.actions[options.closeAction].confirm;
                        if (confirmMsg != undefined && confirmMsg != "")
                            $("a.ui-dialog-titlebar-close").hide();
                    }
                },
                beforeClose: function () {

                    var buttonName = $("#" + anchorName).data("buttonName");
                    var actions = $("#" + anchorName).data("actions");

                    //if the dialog close was clicked and there is a cancel action defined go ahead and close without validation
                    if ((buttonName == undefined || buttonName == "") && options.closeAction != "")
                        return true;

                    //get the action
                    var actionName = "";
                    $.each(actions, function (key, value) {
                        if (key == buttonName) {
                            actionName = key;
                        }
                    });

                    if ($("input[name=WWGAction_" + actionName + "]").attr("data-rsscancel") == "true")
                        return true;

                    var cs = getCurrentScope(this);
                    var valid = fields.validateFields(cs, true);

                    if (!valid)
                        $("#" + anchorName).data("buttonName", "");

                    return valid;
                },
                close: function (e, c) {


                    $(this).dialog("destroy");
                    $(this).dialogExtend("destroy");
                    //$(this).insertAfter("#" + anchorName);	

                    var actions = $("#" + anchorName).data("actions");
                    var buttonName = $("#" + anchorName).data("buttonName");

                    var actionName = "";
                    $.each(actions, function (key, value) {
                        if (key == buttonName) {
                            actionName = key;
                        }
                    });

                    $("#" + anchorName).remove();

                    var btn;
                    if (actionName != "")
                        btn = $("input[name=WWGAction_" + actionName + "]");
                    else if (options.closeAction != "")
                        btn = $("input[name=WWGAction_" + options.closeAction + "]");


                    if (btn != undefined) {
                        //need to setup to skip confirm
                        if (btn.attr("data-rssconfirm") != undefined)
                            btn.attr("data-skipconfirm", "true");
                        btn.click();
                    }

                },
                resizeStop: function (event, ui) {
                    var dialogDiv = $(event.target.parentElement).children("div[data-rssdialog=true]");
                    dialogDiv.attr("data-rssdialog_xpos", ui.position.left);
                    dialogDiv.attr("data-rssdialog_ypos", ui.position.top);
                    dialogDiv.attr("data-rssdialog_width", ui.size.width);
                    dialogDiv.attr("data-rssdialog_height", ui.size.height);
                },
                dragStop: function (event, ui) {
                    var dialogDiv = $(event.target.parentElement).children("div[data-rssdialog=true]");
                    dialogDiv.attr("data-rssdialog_xpos", ui.position.left);
                    dialogDiv.attr("data-rssdialog_ypos", ui.position.top);
                    dialogDiv.attr("data-rssdialog_width", $(event.target.parentElement).width());
                    dialogDiv.attr("data-rssdialog_height", $(event.target.parentElement).height());
            }
        };

            var ops = $.extend(defaultOptions, options.dialogOptions);



            var d = $(selector).dialog(ops).dialogExtend(dialogExtendOptions);


            var ip = $("div#" + anchorName).closest("div.form_div");

            //FIXME:  current scope fails when this line is commented out for child project book dialogs.
            //d.parent(".ui-dialog").appendTo(ip);

            //d.draggable({ containment: "body" });

            //$("div#" + anchorName).closest("div.form_div").append(d.parent(".ui-dialog"));
            //d.parent(".ui-dialog").appendTo("body");
            d.dialog("open");
            d.scrollTop(0);
            //for some reason the auto height is being lost
            if (ops.height == "auto")
                $(selector).css("height", ops.height);

            //$(selector).parent().appendTo($("form:first")); 
        };

        //end ui namespace
        return {
            pageSetup: pageSetup,
            newPageSetup: newPageSetup,
            newPBPageSetup: newPBPageSetup,
            setupUI: setupUI,
            objectClick: objectClick,
            confirmObjectClick: confirmObjectClick,
            openPopup: openPopup,
            cftSubmit: cftSubmit,
            cftSubmitQueue: cftSubmitQueue,
            getAppUrl: getAppUrl,
            matrixClick: matrixClick,
            objectSetFocus: objectSetFocus,
            waitForJSToLoad: waitForJSToLoad,
            waitForAllContent: waitForAllContent,
            getCurrentScope: getCurrentScope,
            setupPBImages: setupPBImages,
            parentChanged: parentChanged,
            getAlertTitle: getAlertTitle,
            waitForPBBranch: waitForPBBranch
        };

    }();


    //------------------------------------------------
    //  Namespace: fields
    //
    //  Public methods:
    //------------------------------------------------
    var fields = function () {

        var validateScript = "";
        var skipNextValidate = false;

        var validateFields = function (cs, fromDialog) {

            if (skipNextValidate) {
                skipNextValidate = false;
                return true;
            }

            if (cs == undefined)
                debugger;

            if (fromDialog == undefined)
                fromDialog = false;


            var errFnd = false;

            cs.find("[data-rssvalidate=true]").each(function () {
                var fieldName = $(this).attr("data-rssfield");
                var field = fieldName.substring(fieldName.lastIndexOf("]") + 1);
                var name = cs.curScope.Fields[field].label;
                var type = cs.curScope.Fields[field].fieldType;
                var canBeNull = cs.curScope.Fields[field].canBeNull;
                var required = cs.curScope.Fields[field].isRequired;
                var regexStr = cs.curScope.Fields[field].regExp;
                var regexMsg = cs.curScope.Fields[field].regExpErrMsg;
                var format = cs.curScope.Fields[field].format;
                var minsize = cs.curScope.Fields[field].minSize * 1;
                var maxsize = cs.curScope.Fields[field].maxSize * 1;

                var focusset = $(this);

                if (type == "Numeric") {
                    var numerror = false;
                    if (canBeNull == "false" && $(this).val().length == 0)
                        numerror = true;

                    if ($(this).val().length > 0 && !isNumeric(this))
                        numerror = true;

                    if (numerror) {
                        RSStudio.err.addErrorMsg(cs.curScope.ErrorMsgs.mustBeNumeric, name);
                        errFnd = true;
                    }
                }

                if (type == "Date and Time") {
                    if ($(this).is("[data-rssdatetime]") && !RSStudioPage.Branch.pageLayout.responsiveDesign) {
                        fieldSel = $(this).data("datepicker");
                    }  else {
                        fieldSel = $(this);
                        format = "yyyy-MM-dd";
                    }

                    var dateErr = false;

                    if (canBeNull == "false" && $(this).val().length == 0)
                        dateErr = true;
                    else if (!isDate(fieldSel.val(), format, cs.curScope.LangTrans.monthNames, cs.curScope.LangTrans.dayNames))
                        dateErr = true;

                    if (dateErr) {
                        RSStudio.err.addErrorMsg(cs.curScope.ErrorMsgs.invalidDate, [name, format]);
                        if (!errFnd) {
                            focusset = fieldSel;
                        }
                        errFnd = true;
                    }
                }

                if (required == "true") {
                    if (type == "Radio Button") {
                        if ($("input[data-rssfield=" + field + "]").filter(":checked").size() == 0) {
                            RSStudio.err.addErrorMsg(cs.curScope.ErrorMsgs.mustSelectOne, name);
                            errFnd = true;
                        }
                    }
                    else if (type == "Dropdown List") {
                        //nothing needs to be done, dropdowns always have a row selected
                    }
                    else {
                        if ($(this).val().length == 0) {
                            RSStudio.err.addErrorMsg(cs.curScope.ErrorMsgs.mustHaveValue, name);
                            errFnd = true;
                        }
                    }

                }

                if (type == "Multiline Text" && (maxsize != 0 || minsize != 0)) {

                    if (maxsize != 0 && $(this).val().length > (maxsize)) {
                        RSStudio.err.addErrorMsg(cs.curScope.ErrorMsgs.maxSize, [name, maxsize]);
                        errFnd = true;
                    }

                    if (minsize != 0 && $(this).val().length < (minsize)) {
                        RSStudio.err.addErrorMsg(cs.curScope.ErrorMsgs.minSize, [name, minsize]);
                        errFnd = true;
                    }

                }



                if (regexStr != "") {
                    var myre = new RegExp(regexStr)
                    if (!myre.test($(this).val())) {
                        RSStudio.err.addErrorMsg(regexMsg);
                        errFnd = true;
                    }

                }

                if (errFnd && $("[data-setfocus]").size() == 0) {
                    if (cs.canFocus(focusset))
                        focusset.attr("data-setfocus", "setfocus");
                }

            });

            validateDynamic(cs);

            if (err.hasErrorsPending()) {
                var alertmsg = err.formatErrorMsg();
                RSStudio.userAlert(alertmsg, function () {
                    err.clearMessages();
                });
                errFnd = true;
            }
            else {

                //run through all child PBs and make sure there are no errors before you submit
                cs.find("span.WWGPBBranch").each(function () {
                    if ($(this).html() != "" && !validateFields(ui.getCurrentScope(this), fromDialog))
                        errFnd = true;
                });
            }


            if (!errFnd && fromDialog)
                skipNextValidate = true;

            return !errFnd;

        };

        var validateDynamic = function (cs) {

            if (cs == undefined)
                debugger;

            //set up page validateion
            if ($("div#RSStudioPageValidate").size() == 0) {
                $("body").append("<div id=\"RSStudioPageValidate\"></div>");
            }

            var script = "<script type=\"text/javascript\" data-scripttype=\"rsstudiovalidate\">function RSStudioPageValidate() {\n";

            if (cs.curScope.Branch.page.hasStandardJS)
                script += "\ttry\n\t{\n\t\tRSStudio_Page_" + cs.curScope.Branch.page.name + ".validate();\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";

            if (cs.curScope.Branch.pageLayout.hasStandardJS)
                script += "\ttry\n\t{\n\t\tRSStudio_PageLayout_" + cs.curScope.Branch.pageLayout.name + ".validate();\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";

            if (cs.curScope.JavaScript.validate != "")
                script += "try\n{\n" + cs.curScope.JavaScript.validate + "\n}\ncatch(e)\n{\n\tRSStudio.err.writeLog(e);\n}\n";

            for (var fld in cs.curScope.Fields) {
                if (cs.curScope.Fields[fld].fieldType == "Custom Field Type" && cs.curScope.Fields[fld].hasStandardJS == "true") {
                    cs.find("[data-rssobject=" + fld + "]").each(function () {
                        if ($(this).html() != "") {
                            var id = $(this).attr("data-rssobject_inst");
                            script += "\ttry\n\t{\n\t\tRSStudio_CFT_" + cs.curScope.Fields[fld].fieldTypeParent + ".validate(\"" + id + "\", \"" + fld + "\");\n\t}\n\tcatch(e)\n\t{\n\t\tRSStudio.err.writeLog(e);\n\t}\n";
                        }
                    });
                }
            }

            script += "}\n";
            script += "//# sourceURL=RSStudioPageValidate.js\n"
            script += "</script>\n";

            if (validateScript != script) {
                validateScript = script;
                $("div#RSStudioPageValidate").html(script);
            }

            RSStudioPageValidate();
        }

        var applyFormat = function (theField) {

            var cs = ui.getCurrentScope(theField);

            var field = $(theField).attr("data-rssfield");
            var baseFieldName = field.substring(field.lastIndexOf("]") + 1);
            var format = cs.curScope.Fields[baseFieldName].format;
            var value = $(theField).val();

            var newvalue = "";
            var idx;
            var formatidx;
            var valchar = "0123456789";
            idx = 0;

            //check for standard values
            if (format == "Upper Case") {
                newvalue = value.toUpperCase();
            }
            else if (format == "Lower Case") {
                newvalue = value.toLowerCase();
            }
            else {
                for (formatidx = 0; formatidx < format.length; formatidx++) {
                    if (idx < value.length) {
                        if (format.charAt(formatidx) == "*") {
                            if (valchar.indexOf(value.charAt(idx)) == -1
			                && value.charAt(idx) != "*") {
                                idx++;
                            }
                            else {
                                newvalue += value.charAt(idx);
                                idx++;
                            }
                        }
                        else if (format.charAt(formatidx) == "#") {
                            if (valchar.indexOf(value.charAt(idx)) == -1) {
                                idx++;
                            }
                            else {
                                newvalue += value.charAt(idx);
                                idx++;
                            }
                        }
                        else {
                            newvalue += format.charAt(formatidx);
                            if (format.charAt(formatidx) == value.charAt(idx)) {
                                idx++;
                            }
                        }
                    }
                }

            }
            if (value != newvalue) {

                $(theField).val(newvalue);

                //the onchange event is buggy at best.  I found various references to both its expected behavior
                //and buggy behavior: http://ewbi.blogs.com/develops/2004/12/ie_changing_a_t.html
                //to get around this, the following makes sure it gets called
                //
                //need to force the onchange method -- this means that for formatted fields we are ok,
                //however this will force a submit at this point
                $(theField).change();

            }
        };

        return {
            applyFormat: applyFormat,
            validateFields: validateFields
        }
    }();

    //------------------------------------------------
    //  Namespace: err
    //
    //  Public methods:
    //------------------------------------------------
    var err = function () {

        var writeLog = function (e) {
            try {
                glbException = e;
                console.log(e);
				console.log(e.stack);
				//console.log(e.message);
                addErrorMsg(e.message);
            }
            catch (ex) {
                addErrorMsg(e.message);
            }
        };

        var addErrorMsg = function (errorMsg, parms) {

            if (parms != undefined) {
                if (!(parms instanceof Array))
                    parms = [parms];

                $.each(parms, function (n, value) {
                    errorMsg = errorMsg.replace("{" + n + "}", value);
                });
            }

            var fnd = false;
            for (var i = 0; i < errorMsgArray.length; i++) {
                if (errorMsgArray[i] == errorMsg) {
                    fnd = true;
                }
            }
            if (!fnd) {
                errorMsgArray.push(errorMsg);
            }
            return;
        };

        var hasErrorsPending = function () {
            if (errorMsgArray.length > 0)
                return true;
            else
                return false;
        };

        var formatErrorMsg = function () {
            var errorMsg = "<span class=\"rss-error-msg\">";
            for (var i = 0; i < errorMsgArray.length; i++) {
                if (i > 0) {
                    errorMsg += "<br><br>";
                }
                errorMsg += errorMsgArray[i];
            }
            errorMsg += "</span>";
            return errorMsg;
        };

        var clearMessages = function () {
            errorMsgArray = new Array();
        };

        return {
            addErrorMsg: addErrorMsg,
            hasErrorsPending: hasErrorsPending,
            formatErrorMsg: formatErrorMsg,
            clearMessages: clearMessages,
            writeLog: writeLog
        };

    }();

    //------------------------------------------------
    //  Namespace: keepAlive
    //
    //  Public methods:
    //      startKeepAliveTimer
    //      pushMessage
    //------------------------------------------------
    var keepAlive = function () {

        var keepAliveID = 0;
        var lastSubmit = 0;

        var startKeepAliveTimer = function () {

            if (RSStudioPage.PageControl.sessionTimeout == "no") {
                if (keepAliveID != 0) {
                    lastSubmit = 0;
                }
                else {
                    keepAliveID = setInterval(function () {
                        RSStudio.keepAlive.pushMessage();
                    }, 1000 * 60);
                }
            }
            else {
                RSStudioPage.PageControl.sessionTimeout = "no";
            }
        };

        var setRefreshTime = function (cs) {

            var dt = (new Date).getTime();
            if (cs.isBody)
                cs.curScope.PageControl.lastSubmit = dt;
            else {
                $.each(RSStudioPage.PBBranch, function (idx, value) {
                    if (value.PageControl.projectBookIdent == cs.curScope.PageControl.projectBookIdent) {
                        value.PageControl.lastSubmit = dt;
                    }
                });

            }
        };

        var sendKeepAliveMessage = function (pc, url, curDate) {


            if (pc.lastSubmit == 0) {
                //assume that activity occured minute before
                pc.lastSubmit = curDate - 60 * 1000;
            }
            else {

                if (pc.lastSubmit + 1000 * 60 * 3 <= curDate) {
                    pc.lastSubmit = curDate;

                    $.ajax({
                        url: url,
                        type: "POST",
                        dataType: "json",
                        data: { requestType: "keepAlive" },
                        cache: false,
                        success: function (e, c) {
                            var resp;
                        },
                        error: function (e, c) {
                            var resp;
                        }
                    });
                }
            }
        };

        var pushTimeout = function () {


            $.ajax({
                url: RSStudioPage.Ajax.infoExch,
                type: "POST",
                dataType: "json",
                data: { requestType: "timeout" },
                cache: false,
                success: function (e, c) {

                    var resp;
                    location.href = "WWGStart"
                },
                error: function (e, c) {

                    var resp;
                    location.href = "WWGStart"
                }
            });
        };

        var pushMessage = function () {


            lastSubmit += 1;

            if (lastSubmit >= RSStudioPage.Ajax.timeoutMinutes) {
                clearInterval(keepAliveID);
                keepAliveID = 0;
                lastSubmit = 0;
                pushTimeout();
            }
            else {
                var curDate = (new Date).getTime();

                sendKeepAliveMessage(RSStudioPage.PageControl, RSStudioPage.Ajax.infoExch, curDate);

                var pbSession = new Array();

                $.each(RSStudioPage.PBBranch, function (idx, value) {

                    var dbPath = value.PageControl.dbPath;
                    if (indexOf(pbSession, dbPath) == -1) {
                        pbSession.push(dbPath);

                        var url = "/" + dbPath + "/" + RSStudioPage.Ajax.infoExch;
                        sendKeepAliveMessage(value.PageControl, url, curDate);
                    }
                    else {
                        if (value.PageControl.lastSubmit + 1000 * 60 * 3 <= curDate)
                            value.PageControl.lastSubmit = curDate;
                    }
                });



                //if its going to expire in 2 min then put up message if there is one
                if (lastSubmit + 2 == RSStudioPage.Ajax.timeoutMinutes && RSStudioPage.Ajax.timeoutMsg != "") {
                    RSStudio.userAlert(RSStudioPage.Ajax.timeoutMsg, function () {
                        lastSubmit = 0;
                    });
                }
            }
        };


        return {
            startKeepAliveTimer: startKeepAliveTimer,
            pushMessage: pushMessage,
            setRefreshTime: setRefreshTime
        };

    }();

    var events = function () {

        var listenEventID = 0;

        var listenForEvents = function () {

            if (listenEventID == 0) {
                listenEventID = 1;
                listenSubmit();
            }
        };

        var listenSubmit = function () {


            $.ajax({
                url: "WWGEventListen",
                type: "POST",
                dataType: "xml",
                data: {
                    status: "listen",
                    WWGSessionIdx: RSStudioPage.PageControl.sessionIdx,
                    WWGThreadIdx: RSStudioPage.PageControl.threadIdx
                },
                cache: false,
                success: function (xml) {
                    listenSubmitDone(xml);
                },
                error: function (xml) {
                    listenSubmitDone(xml);
                }
            });

        };

        var listenSubmitDone = function (xml) {

            var node = getNodeByTagName(xml, "WWGListen");
            var listenNode = getNodeByTagName(node, "ListenForEvents");
            var execEvent = getNodeByTagName(node, "ExecuteEvent");
            if (listenNode != null) {
                listenSubmit("status=listen");
            }
            else if (execEvent != null) {
                EventSubmit();
            }
            else {
                listenEventID = 0;
            }
        };



        var EventSubmit = function () {

            ajax.lockPage();

            $.ajax({
                url: "WWGEventExec",
                type: "POST",
                dataType: "xml",
                data: {
                    WWGSessionIdx: RSStudioPage.PageControl.sessionIdx,
                    WWGThreadIdx: RSStudioPage.PageControl.threadIdx
                },
                cache: false,
                async: false,
                success: function (xml) {
                    eventSubmitDone(xml);
                },
                error: function (xml) {
                    eventSubmitDone(xml);
                }
            });

        };


        var eventSubmitDone = function (xml) {

            listenEventID = 0;


            var node = getNodeByTagName(xml, "WWGEventResponse");

            var branchNode = getNodeByTagName(node, "BranchNeeded");

            var cs = ui.getCurrentScope($("body"));
            var errorFnd = ajax.processXmlHttpResponse(node, cs);

            if (branchNode != null) {

                cs.curScope.PageControl.objectType = "";
                cs.curScope.PageControl.objectValue = "";
                cs.curScope.PageControl.objectParent = "";
                cs.curScope.PageControl.objectRow = "";
                cs.curScope.PageControl.objectCol = "";
                cs.curScope.PageControl.branchFlag = "0";

                ajax.submit(cs);
            }

            ajax.unlockPage();

        };

        return {
            listenForEvents: listenForEvents
        };

    }();

    //------------------------------------------------
    //  Namespace: ajax
    //
    //  Public methods:
    //      submit
    //
    //  Private methods:
    //      encodeURL
    //------------------------------------------------
    var ajax = function () {

        var encodeUrl = function (cs, skipProcessForm) {

            if (cs == undefined)
                debugger;

            if (cs.curScope == undefined)
                debugger;


            var vals = cs.find(":input").serializeArray();


            //add checkbox fields that are not checked since they are only included as "on" when checked
            cs.find("[data-rssfield]:checkbox").not(":checked").each(function () {
                vals.push({ name: $(this).attr("name"), value: "false" });
            });

            //make sure that each matrix is in the list at least once
            cs.find("table[data-rssmatrix], div[data-rssmatrix], fieldset[data-rssmatrix]").closest("div[data-rssobject_inst]").each(function () {
                vals.push({ name: $(this).attr("data-rssobject_inst"), value: "" });
            });

            //make sure that row selector fields are in the list at least once
            cs.find("table[data-rssfield][data-rssfield_selector=Row]").closest("div[data-rssobject_inst]").each(function () {
                vals.push({ name: $(this).attr("data-rssobject_inst"), value: "" });
            });

            cs.find("ul[data-rssfield][data-rssfield_selector=Listview]").closest("div[data-rssobject_inst]").each(function () {
                vals.push({ name: $(this).attr("data-rssobject_inst"), value: "" });
            });

            cs.find("ul[data-rssmatrix][data-rssmatrix_selector=Listview]").closest("div[data-rssobject_inst]").each(function () {
                vals.push({ name: $(this).attr("data-rssobject_inst"), value: "" });
            });


            var retVals = new Array();

            $(vals).each(function (idx) {
                var name = this.name;
                var val = this.value;

                var elm = cs.find("[data-rssfield][name='" + name + "']");

                if (elm.size() > 0) {
                    name = elm.attr("data-rssfield");
                    if (elm.is(":checkbox") && val == "on") {
                        val = "true";
                    } else if (elm.is("table[data-rssfield][data-rssfield_selector=Row]")) {
                            val = elm.find("tr[data-rssfield_row=cur]>td").attr("data-dbvalue");
                    } else if (elm.is("ul[data-rssfield][data-rssfield_selector=Listview]")) {
                            val = elm.find("li[data-rssfield_row=cur]").attr("data-dbvalue");
                    } else if (elm.is("[data-rssdatetime]")) {
                        if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {

                            val = $.datepicker.formatDate("yy-mm-dd", elm.data("datepicker").datepicker("getDate"));
                            if (val != "") {
                                var theDate = elm.data("datepicker").datepicker("getDate");
                                var hr = theDate.getHours();
                                var mn = theDate.getMinutes();
                                var s = theDate.getSeconds();
                                val += $.datepicker.formatTime(' HH:mm:ss', { hour: hr, minute: mn, second: s });
                            }
                        } else {
                            val += " 00:00:00";
                        }
                    }
                }
                else {
                    elm = cs.find("[data-rssobject_inst='" + name + "']").find(">table[data-rssmatrix], >ul[data-rssmatrix], >select[data-rssmatrix], >div[data-rssmatrix], div.tablescroll_wrapper>table[data-rssmatrix], >fieldset[data-rssmatrix]");

                    if (elm.size() == 0 && RSStudioPage.Branch.pageLayout.responsiveDesign) {
                        elm = cs.find("[data-rssobject_inst='" + name + "']").find(">div.ui-select>div.ui-btn>select[data-rssmatrix]");
                    }

                    if (elm.size() == 0)
                        elm = cs.find("[data-rssobject_inst='" + name + "']").find("select[data-rssmatrix]:first-child");

                    if (elm.size() > 0) {

                        var instName = name;
                        name = elm.attr("data-rssmatrix");
                        if (findInList(retVals, "name", name) == -1) {
                            var holdval = val;
                            val = "";

                            var inst = cs.find("[data-rssobject_inst='" + instName + "']");
                            var hasChanged = inst.attr("data-has_changed");
                            inst.removeAttr("data-has_changed");

                            if (hasChanged == undefined) {
                                val = "";
                            }
                            else if (elm.is("select")) {
                                elm.find("option").each(function () {
                                    if ($(this).val() == holdval)
                                        val += "C";
                                    else if ($(this).val() != "-1")
                                        val += " ";
                                });
                            }
                            else if (elm.attr("data-rssmatrix_selector") == "Accordion") {
                                elm.find(">h2").each(function () {
                                    if ($(this).is("[data-rssmatrix_row=cur]"))
                                        val += "C";
                                    else
                                        val += " ";
                                });
                            }
                            else if (elm.attr("data-rssmatrix_selector") == "Tab") {
                                var selector = ">ul>li";
                                if (RSStudioPage.Branch.pageLayout.responsiveDesign) {
                                    selector = ">div>ul>li";
                                }
                                elm.find(selector).each(function () {
                                    if ($(this).is("[data-rssmatrix_row=cur]"))
                                        val += "C";
                                    else if ($(this).is("[data-rssmatrix_row=del]"))
                                        val += "D";
                                    else
                                        val += " ";
                                });
                            }
                            else if (elm.attr("data-rssmatrix_selector") == "Row") {
                                elm.find(">tbody>tr.ui-selectee").each(function () {
                                    if ($(this).hasClass("ui-selected")) {
                                        if ($(this).is("[data-rssmatrix_row=cur]"))
                                            val += "C";
                                        else
                                            val += "M";
                                    }
                                    else {
                                        val += " ";
                                    }
                                });
                            }
                            else if (elm.attr("data-rssmatrix_selector") == "No Selector") {
                                elm.find(">tbody>tr").each(function () {
                                    if ($(this).is("[data-rssmatrix_row=cur]"))
                                        val += "C";
                                    else
                                        val += " ";
                                });
                            } else if (elm.attr("data-rssmatrix_selector") == "Listview") {
                                elm.find(">li").each(function () {
                                    if ($(this).is("[data-theme]")) {
                                        if ($(this).is("[data-rssmatrix_row=cur]"))
                                            val += "C";
                                        else
                                            val += "M";
                                    }
                                    else {
                                        val += " ";
                                    }
                                });
                            }
                            else {
                                elm = elm.find("input[name^='" + instName + "']");
                                if (elm.is(":radio") || elm.is(":checkbox")) {
                                    elm.each(function (idx) {
                                        if ($(this).is(":checked")) {
                                            if ($(this).closest("tr").is("[data-rssmatrix_row=cur]"))
                                                val += "C";
                                            else if ( $(this).siblings("label").is("[data-rssmatrix_row=cur]") )
                                                val += "C";
                                            else if ($(this).is(":checkbox"))
                                                val += "M";
                                            else
                                                val += " ";
                                        }
                                        else {
                                            val += " ";
                                        }
                                    });
                                }
                            }
                        }

                    }
                    else if (cs.find("input[name='" + name + "']").hasClass("hasDatepicker")) {
                        name = "";
                    }
                }

                if (name != "" && findInList(retVals, "name", name) == -1)
                    retVals.push({ name: name, value: val });

            });

            retVals.push({ name: "WWGBranchFlag", value: cs.curScope.PageControl.branchFlag });
            retVals.push({ name: "WWGSessionIdx", value: cs.curScope.PageControl.sessionIdx });
            retVals.push({ name: "WWGThreadIdx", value: cs.curScope.PageControl.threadIdx });
            retVals.push({ name: "WWGSubmitCounter", value: cs.curScope.PageControl.submitCounter });
            retVals.push({ name: "WWGObjectType", value: cs.curScope.PageControl.objectType });
            retVals.push({ name: "WWGObjectParent", value: cs.curScope.PageControl.objectParent });
            retVals.push({ name: "WWGObjectValue", value: cs.curScope.PageControl.objectValue });
            retVals.push({ name: "WWGObjectRow", value: cs.curScope.PageControl.objectRow });
            retVals.push({ name: "WWGObjectCol", value: cs.curScope.PageControl.objectCol });
            retVals.push({ name: "WWGSessionKey", value: cs.curScope.PageControl.sessionKey });


            //clear variables
            cs.curScope.PageControl.objectType = "";
            cs.curScope.PageControl.objectValue = "";
            cs.curScope.PageControl.objectParent = "";
            cs.curScope.PageControl.objectRow = "";
            cs.curScope.PageControl.objectCol = "";
            cs.curScope.PageControl.branchFlag = "";

            if (skipProcessForm)
                retVals.push({ name: "WWGSkipFormProcess", value: "true" });


            return retVals;
        };

        var forceChildSubmit = function (cs) {

            //recurse through children
            $.each(cs.find("span.WWGPBBranch[data-has_changed='changed']"), function (index, value) {
                $(this).attr("data-has_changed", "");
                var childcs = ui.getCurrentScope(this);
                forceChildSubmit(childcs);
                submitValues(childcs, false, true);
            });


        };

        var processXmlHttpResponse = function (node, cs) {

            if (cs == undefined)
                debugger;

            var alertNode = getNodeByTagName(node, "AlertMessage");
            var pageLoadJS = getNodeByTagName(node, "PageLoadJS");
            var validationJS = getNodeByTagName(node, "ValidationJS");
            var exceptionNode = getNodeByTagName(node, "Exception");
            var listenNode = getNodeByTagName(node, "ListenForEvents");
            var sessionKeyError = getNodeByTagName(node, "SessionKeyError");
            var errorFnd = false;



            //make sure the scope is current.
            if (!cs.isBody) {

                var newPB = $("body").find("span.WWGPBBranch[data-pbpath=" + cs.curObj.attr("data-pbpath") + "][data-pbsessionidx=" + cs.curScope.PageControl.sessionIdx + "]");
                cs = ui.getCurrentScope(newPB);
            }


            node = getNodeByTagName(node, "Objects");
            var obj = getFirstChildNode(node);
            while (obj != null) {

                try {

                    var rssobject_inst = cs.find("[data-rssobject_inst=" + obj.nodeName + "],[data-rssobject_inst=" + obj.nodeName + "_" + cs.curScope.PageControl.sessionKey + "]");


                    //make sure the object was not completely removed from the dom
                    if (rssobject_inst.size() != 0) {

                        //for mobile we don't want to set this
                        if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {
                            var lastHeight = rssobject_inst.attr("data-lastheight") == undefined ? 0 : rssobject_inst.attr("data-lastheight") * 1;

                            if (rssobject_inst.css("min-height") == "0px"
                            || rssobject_inst.height() > lastHeight + 20) {
                                rssobject_inst.css("min-height", rssobject_inst.height());
                                rssobject_inst.attr("data-lastheight", rssobject_inst.height());
                            }
                        }

                        rssobject_inst.html(getNodeValue(obj));

                        if (!cs.isBody)
                            RSStudio.ui.setupPBImages(rssobject_inst, cs.curScope);

                    } else {

                        writeConsole("zero length object found:" + "[data-rssobject_inst=" + obj.nodeName + "],[data-rssobject_inst=" + obj.nodeName + "_" + cs.curScope.PageControl.sessionKey + "]");

                    }
                }
                catch (e) {
                    writeConsole("Error on replacement of HTML:" + "[data-rssobject_inst=" + obj.nodeName + "],[data-rssobject_inst=" + obj.nodeName + "_" + cs.curScope.PageControl.sessionKey + "]");
                    errorFnd = true;
                }

                obj = getNextSiblingNode(obj);
            }




            if (pageLoadJS != null) {
                cs.curScope.JavaScript.pageLoad = getNodeValue(pageLoadJS);
            }


            if (exceptionNode != null) {
                RSStudioPage.PageControl.alertMessage = getNodeValue(exceptionNode);
            }
            else if (alertNode != null) {
                RSStudioPage.PageControl.alertMessage = getNodeValue(alertNode);
            }
            else if (sessionKeyError != null) {
                RSStudioPage.PageControl.alertMessage = getNodeValue(sessionKeyError);
            }

            if (validationJS != null) {
                cs.curScope.JavaScript.validate = getNodeValue(validationJS);

            }

            ui.setupUI(cs.curObj);


            if (listenNode != null) {
                events.listenForEvents();
            }
            else {
                listenEventID = 0;
            }

            return errorFnd;
        };


        var submit = function (cs, skipFormProcess) {


            if (cs == undefined)
                debugger;

            if (skipFormProcess == undefined)
                skipFormProcess = false;

            forceChildSubmit(cs);

            //allow all external events to finish first! Do not comment this out.
           	setTimeout(function() {

            	submitValues(cs, skipFormProcess, false);

            }, 1);

        };

        var submitNoWait = function (cs, skipFormProcess) {

            if (cs == undefined)
                debugger;

            if (skipFormProcess == undefined)
                skipFormProcess = false;

            forceChildSubmit(cs);
            submitValues(cs, skipFormProcess, false);


        };


        var submitValues = function (cs, skipFormProcess, skipResults) {

            //skipResults when this is a recursive uphill call to get children to submit.
            lockPage();

            var vals;

            vals = encodeUrl(cs, skipFormProcess);

            var url = cs.curScope.Ajax.ajaxURL;
            if (cs.curObj.is("span.WWGPBBranch")) {
                //clear data change flag so we don't process twice
                cs.curObj.attr("data-has_changed", "false");
                url = "/" + cs.curObj.attr("data-pbpath") + "/" + url;

            }

            keepAlive.setRefreshTime(cs);



            $.ajax({
                url: url,
                type: "POST",
                dataType: "xml",
                data: vals,
                cache: false,
                async: cs.isBody ? false : true,
                success: function (xml) {

                    if (!skipResults)
                        submitDone(xml, cs);
                    else
                        unlockPage();
                },
                error: function (xml) {

                    if (!skipResults)
                        submitDone(xml, cs);
                    else
                        unlockPage();
                }
            });


        };

        var openLockDialogTimer = 0;
        var openLockDialogTime = 1000;
        var openLockDialog = function () {



            if ($("#RSStudioLockDialog").dialog("isOpen") == true) {
                return;
            }


            //mark all existing overlays
            $("div.ui-widget-overlay").each(function (idx, e) {
                $(this).data("jquery-dialog-overlay", true);
                $(this).data("jquery-dialog-overlay-op", $(this).css("background"));
            });

            if ($("div#RSStudioLockDialog").size() == 0) {
                $("body").append("<div id=\"RSStudioLockDialog\"><img src=images/ajax-loader.gif></div>");
            }

            var dlg = $("#RSStudioLockDialog").dialog({
                autoOpen: false,
                modal: true,
                closeOnEscape: false,
                draggable: false,
                resizable: false

            });

            dlg.parent().find("div.ui-dialog-titlebar").remove();

            dlg.dialog("open");

            dlg.parent().css("height", "65px");
            dlg.parent().css("width", "80px");

            dlg.parent().hide();

            //find overlay
            var ovl;
            $("div.ui-widget-overlay").each(function (idx, e) {
                if ($(this).data("jquery-dialog-overlay") != true)
                    ovl = $(this);
            });

            if (ovl != undefined) {
                ovl.data("jquery-dialog-overlay-op", ovl.css("background"));
                ovl.css("background", "transparent");
            }

            openLockDialogTimer = setTimeout(function () {

                openLockDialogTimer = 0;
                if (dlg.dialog("isOpen") == true) {

                    ovl.css("background", ovl.data("jquery-dialog-overlay-op"));
                    dlg.parent().show();

                }
            }, openLockDialogTime);
        };

        var setOpenLockDialogWaitTime = function (waitTime) {
            openLockDialogTime = waitTime;
        }

        var closeLockDialog = function () {


            if (openLockDialogTimer != 0) {
                clearTimeout(openLockDialogTimer);
                openLockDialogTimer = 0;
            }

            $("div.ui-widget-overlay").each(function (idx, e) {
                $(this).css("background", $(this).data("jquery-dialog-overlay-op"));
            });

            if ($("#RSStudioLockDialog").dialog("isOpen") == true) {
                $("#RSStudioLockDialog").dialog("close");

            }


        };

        var messageQueue = new Array();

        var queueMessages = function (messageListNode) {

            if (messageListNode == null) {
                return;
            }

            var msg = getFirstChildNode(messageListNode);
            while (msg != null) {

                var url = getNodeByTagName(msg, "url");
                var qs = getNodeByTagName(msg, "qs");
                var si = getNodeByTagName(msg, "si");

                messageQueue.push({ url: getNodeValue(url), queryString: getNodeValue(qs), sessionIdx: getNodeValue(si) });

                msg = getNextSiblingNode(msg);
            }

        };


        var processMessageWait = false;
        var processMessages = function () {
            if (messageQueue.length > 0 && !processMessageWait) {
                processMessageWait = true;
                var msg = messageQueue.shift();
                if (msg.isBody) {
                    sendMessageChild(msg);
                } else {
                    sendMessage(msg);
                }
            }
        };

        var submitDone = function (xml, cs) {


            if (cs == undefined)
                debugger;

            var node = getNodeByTagName(xml, "WWGAJAXResponse");
            var redirect = getNodeByTagName(node, "Redirect");
            var response = getNodeByTagName(node, "Response");
            var newpage = getNodeByTagName(node, "NewPage");
            var fileStreamOpen = getNodeByTagName(node, "WWGFileStreamOpen");
            var messageListNode = getNodeByTagName(node, "RSMessageList");

            if (fileStreamOpen != null) {
                var url = "WWGFileStream";
                if (cs.curObj.is("span.WWGPBBranch")) {
                    url = "/" + cs.curObj.attr("data-pbpath") + "/" + url;
                    //url = "/" + cs.curObj.attr("data-pbpath") + "/test.pdf";
                }

                //$(window).unbind('beforeunload');
                window.open(url);
            }

            //queue up messages
            queueMessages(messageListNode);

            if (node == null) {
                RSStudio.userAlert("Error: Internet connection lost. Please try again.");

            }
            else if (newpage != null) {
                var studioPage = getNodeByTagName(node, "RSStudioPage");

                if (cs.isBody) {
                    RSStudioPage = $.parseJSON(getNodeValue(studioPage));
                    RSStudioPage.PageConrol.lastSubmit = (new Date).getTime();
                    RSStudioPage.PageConrol.dbPath = "";
                }
                else {
                    var sessionIdx = cs.curObj.attr("data-pbsessionidx");
                    var dbpath = cs.curObj.attr("data-pbpath");
                    RSStudioPage.PBBranch[dbpath + sessionIdx] = $.parseJSON(getNodeValue(studioPage));
                    RSStudioPage.PBBranch[dbpath + sessionIdx].PageControl.lastSubmit = (new Date).getTime();
                    RSStudioPage.PBBranch[dbpath + sessionIdx].PageControl.dbPath = dbpath;
                    RSStudioPage.PBBranch[dbpath + sessionIdx].PageControl.executePageLoad = true;
                }

                cs.curObj.html(getNodeValue(newpage));
                if (cs.isBody)
                    ui.newPageSetup();
                else
                    ui.newPBPageSetup(cs);

            }
            else if (redirect != null) {
                unlockPage(true);

                location.href = getNodeValue(redirect);
            }
            else if (response != null) {
                document.body.innerHTML = getNodeValue(response);
            }
            else {
                var fieldList = getNodeByTagName(node, "FieldList");
                $.extend(cs.curScope.Fields, $.parseJSON(getNodeValue(fieldList)));
                var errorFnd = processXmlHttpResponse(node, cs);
                if (errorFnd) {
                    unlockPage(true);
                    location.href = location.href;
                }
            }

            processMessages();

            unlockPage();
        };

        var sendMessageChild = function (msg) {

            var url = msg.url;
            var vals = msg.queryString;
            var sessionIdx = msg.sessionIdx;

            var pbName = url.substring(1, url.lastIndexOf("/"));

            var selector = "span.WWGPBBranch[data-pbsessionidx=" + sessionIdx + "][data-pbpath=" + pbName + "]";


            ui.waitForPBBranch({

                selector: selector,
                success: function () {

                    sendMessage(msg);
                },
                error: function () {
                    processMessageWait = false;
                    writeConsole("Unable to send message to child PB:" + selector);
                }
            });



        };

        var sendMessage = function (msg) {

            processMessageWait = false;
            var url = msg.url;
            var vals = msg.queryString;
            var sessionIdx = msg.sessionIdx;

            lockPage();
            $.ajax({
                url: url,
                type: "POST",
                dataType: "xml",
                data: vals,
                cache: false,
                async: true,
                success: function (xml) {
                    sendMessageDone(xml, url, sessionIdx);
                },
                error: function (xml) {
                    sendMessageDone(xml, url, sessionIdx);
                }
            });

        };

        var sendMessageDone = function (xml, url, sessionIdx) {


            var pbName = url.substring(1, url.lastIndexOf("/"));
            var selector = "span.WWGPBBranch[data-pbsessionidx=" + sessionIdx + "][data-pbpath=" + pbName + "]";

            if ($(selector).size() > 0) {

                sendMessageDoneChild(selector);
            } else {

                var cs = ui.getCurrentScope($("body"));
                submitNoWait(cs, true);
                unlockPage();
            }
        }

        var childMsgStack = new Array();
        var sendMessageDoneChild = function (selector) {
            //need to wait for child to finish loading.

            var num = childMsgStack.length;

            childMsgStack.push(selector);

            if (num == 0)
                sendMessageDoneChildStack();
        }

        var sendMessageDoneChildStack = function () {

            var selector = childMsgStack[0];

            ui.waitForPBBranch({

                selector: selector,
                success: function () {
                    childMsgStack.shift();
                    if (childMsgStack.length > 0) {
                        setTimeout(function () {
                            sendMessageDoneChildStack();
                        }, 10);
                    }

                    var cs = ui.getCurrentScope(selector);
                    submitNoWait(cs, true);
                    unlockPage();

                },
                error: function () {
                    writeConsole("Unable to resubmit child PB after message complete process:" + selector);
                    childMsgStack.shift();
                    unlockPage();
                    if (childMsgStack.length > 0) {
                        sendMessageDoneChildStack();
                    }

                }
            });

        }

        //this method is used to keep the lock dialog open but it does not prevent items from running
        var screenLock = 0;

        var lockScreen = function () {
            screenLock++;

        };

        var unlockScreen = function () {
            screenLock--;
            if (RSStudioPage.PageControl.pageLock == 0 && screenLock == 0)
                closeLockDialog();


        };

        var lockPage = function () {

            if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {
                openLockDialog();
            } else {
                //$.mobile.loading('show');
            }


            document.body.style.cursor = 'wait';
            RSStudioPage.PageControl.pageLock += 1;

            //don't allow the browser to close in the middle of an ajax request

            if ($(window).data("WWGCustomUnload") != true) {
                $(window).bind('beforeunload', function () {
                    return '';
                });
            }
        };

        var unlockPage = function (force) {

            if (force == undefined)
                force = false;

            if (force) {

                RSStudioPage.PageControl.pageLock = 0;
            }
            else {
                RSStudioPage.PageControl.pageLock -= 1;
            }


            if (RSStudioPage.PageControl.pageLock == 0) {
                document.body.style.cursor = '';

                if ($(window).data("WWGCustomUnload") != true)
                    $(window).unbind('beforeunload');

                if (screenLock == 0) {

                    if (!RSStudioPage.Branch.pageLayout.responsiveDesign) {
                        closeLockDialog();
                    } else {
                        //$.mobile.loading('hide');
                    }

                }
            }

        };

        var eventLockFlag = false;

        var eventLock = function () {
            eventLockFlag = true;
        };
        var eventUnlock = function () {
            eventLockFlag = false;
        };
        var isEventLock = function () {
            return eventLockFlag;
        };


        //end of ajax namespace
        return {
            lockPage: lockPage,
            unlockPage: unlockPage,
            submit: submit,
            submitNoWait: submitNoWait,
            processXmlHttpResponse: processXmlHttpResponse,
            lockScreen: lockScreen,
            unlockScreen: unlockScreen,
            eventLock: eventLock,
            eventUnlock: eventUnlock,
            isEventLock: isEventLock,
            setOpenLockDialogWaitTime: setOpenLockDialogWaitTime,
            queueMessages: queueMessages,
            processMessages: processMessages
        };

    }();


    //public RSStudio
    return {
        ui: ui,
        userAlert: userAlert,
        userConfirm: userConfirm,
        fields: fields,
        keepAlive: keepAlive,
        err: err,
        ajax: ajax,
        events: events,
        glbException: glbException
    };

}();

